#  TMC_seg000_Start_Test.R 
#  (c) wwosniok@math.uni-bremen.de

#  Truncated minimum chi-square estimation
#  For method description see:
#  W. Wosniok, R. Haeckel: A new indirect estimation of reference intervals:
#  truncated minimum chi-square (TMC) approach. 
#  https://doi.org/10.1515/cclm-2018-1341

#  For the programme manual: see the text in the 'text' directory

#  Template starting programme, also used for testing the installation.
#  For regular analysis: User fills this file according to his/her needs 
#  and starts execution from here.
#  NOTE: Everything after a # in a line is a comment.
#        Comments are informations to the reader only, the R programme does 
#        nothing with them.

#  The description of the data file is contained in DataFileInfo.csv.
#  The start programme is linked to the data description by the FileNo (below). 
#  The location of DataFileInfo.csv is defined in TMC_seg010_User.R

#  The user name for this sample is 'Usr'. The user name is also set in 
#  TMC_seg010_User.R . The output is tailored for this user.
 

#  #  CHANGE HISTORY
#  25.04.2021 Start

# ===========================================================================
# Read external files

source("TMC_seg010_User.R")            # clean up & define user characteristics
source("TMC_seg015_DefaultSettings.R") # Default parameters

# ==========================================================================
#  Start of user input.

# --------------------------------------------------------------------------
#  Choose the input data set

#  (1) Identification number of the data set (refer to DataFileInfo.csv) 
FileNo <-  9999   # Test data, similar to AST

#  (2)  Scaling factor. Input data will be multiplied by this factor,
#       1 = no change.
scale.fact <- 1

# --------------------------------------------------------------------------
#  Filter the input data

# (3) Exclude extreme values from all calculations
#     Limits apply to scaled values 
#     NA: no exclusion
x.lo.limit <-    NA         # Exclude values < x.lo.limit
x.hi.limit <-    NA         # Exclude values > x.hi.limit

#  (4) Select outpatient / hospitalized subset from data
#      Only valid, if a column for the variable 'OH' is given in 
#      DataFileInfo.csv
#      If the data contains no variable 'OH' or no filtering is desired,
#      set use.oh below to NA  
use.oh  <- NA

#  (5) Select device subset for analysis
#      Only valid, if a column for the variable 'Device' is given in 
#      DataFileInfo.csv
#      If the data contains no variable 'Device' or no filtering is desired,
#      set use.dev below to NA  
use.dev  <- NA

#  (6) Daytime selection: use only data obtained between  ... 
#      (no selection: set all 4 values to NA)
ana.hour.min <- NA      #  Example: 06
ana.minu.min <- NA      #  Example: 30

#  (7) and ...
ana.hour.max <- NA      #  Example: 18
ana.minu.max <- NA      #  Example: 45 

#  (8) Weekday selection:
use.wday <- "all"       #  "all"   or a subset of 
                        #  c("mo", "tu", "we", "th", "fr", "sa", "su")
                        #  use.wday <- c("sa", "su")

#  (9) Date selection: Split the data at a given date and time and select
#      the part to analyse
#      (no selection: set all three variables to NA)
split.date <- NA        # Example: "04.02.2019", format is "dd.mm.yyyy"
split.time <- NA        # Example: "13:25",      format is "hh:mm"
use.split  <- NA        # 1: before /  2: after given date-time

# (10) Define age intervals
#      Provide the left limits of the intervals.
#      The left limits belong to the interval, the right not, except
#      for the rightmost interval, which includes also the right limit
#      Example: c(18, 30, 40, 50) defines intervals 18-29, 30-39, 40-50
#      age.limits <-  c(NA, NA) means to use all data, without age separation
age.limits <- c(18, 30, 40, 50, 60, 70, 80, 90, 100)

# --------------------------------------------------------------------------
#  Specify calculation details

# (11) Probability levels for the RLs 
RL1.p <- 0.025
RL2.p <- 0.975
 
# --------------------------------------------------------------------------
#  Control the appearance of plots

#  (12) Define lower and upper x limits for better readable detail plots
#       of the fitted distribution.
#       These limits affect only the display, not the calculation.
#       Limits apply to scaled values.
x.clip.min   <-      0    # left limit
x.clip.max   <-     50    # right limit
x.clip.by1   <-      5    # horizontal tickmarks with labels
x.clip.by2   <-      1    # horizontal tickmarks without labels
                      
                      
# (13) x limits and tick marks for the age scale in plots of age vs RL
#      These plots (no. 150.010 and 160.010) are produced only if at least 
#      4 age groups exist. 
age.clip.min   <-       10  # left limit
age.clip.max   <-      100  # right limit
age.clip.by1   <-       10  # horizontal tickmarks with labels
age.clip.by2   <-        5  # horizontal tickmarks without labels

# (14) y limits and tick marks for the RL scale in plots of 
#      age vs RL.
#      These plots (no. 150.010 and 160.010) are produced only if at least 
#      4 age groups exist.
#      Set to NA for automatic selection. 
 
RL.clip.min   <-      0  # lower limit
RL.clip.max   <-     45  # upper limit
RL.clip.by1   <-      5  # vertical tickmarks with labels
RL.clip.by2   <-      1  # vertical tickmarks without labels

# --------------------------------------------------------------------------
#  If this is the analysis of a file sequence (usually simulated data):
#  Specify the sequence (otherwise set both variables to NA)

#  (15) First and last replicate
r.start     <-   NA                     # Example:   1
r.ende      <-   NA                     # Example: 100

# --------------------------------------------------------------------------
####  Default settings are in      TMC_seg015_DefaultSettings.R
####  Put changes of default settings after this line 

#  Example of user modification: the default operation is to fit a 
#  log-normal distribution, which is equivalent to fitting a power normal
#  distribution with shape parameter lambda = 0. 
#  To fit a more general distribution, the programme can be instructed to 
#  estimate the optimal lambda from the range [0, 1], where lambda = 1
#  corresponds to a Gaussian (normal) distribution. 
#  Removing the '#' in the two lines below causes the estimation of lambda
#  in [0, 1]

#  lambda.min <- 0
#  lambda.max <- 1

#  Similarly, the following commands force fitting a normal distribution. 
#  lambda.min <- 1
#  lambda.max <- 1

# ---------------------------------------------------------------------
#  End of user input. 
# --------------------------------------------------------------------------

# =================================================================
#  Start execution
# =================================================================

source("TMC_seg050_Master.R")      # Master file, controls execution

# =================================================================
# =====  Programme end  ===========================================
# =================================================================
