#  TMC_seg050_Master.R 
#  (c) wwosniok@math.uni-bremen.de

#  Truncated minimum chi-square estimation
#
#  Master segment. Organises the analysis:
#  - Interpret user input
#  - Set up output files
#  - Do data analysis per subgroup
#  - Produce summary output

#  Naming convention

#  Output files are either plot files (directory figs) or
#  tables/ text files (directory tabs).
#  Output files refer either 
#  - to a stratum (=combination of sex and age) - these go in subdirectory 
#    1_stra under figs or tabs, respectively
#  - to a file (=complete data file) - these go in subdirectory 
#    2_file under figs or tabs, respectively
#  - to a sequence of files (a scenario, typically from a simulation) - these go in 
#    subdirectory 3_sce under figs or tabs, respectively

#  File names have the form outname_suffix, where
#  outname <- paste(file name, column number of measurand in the data file, 
#                   subset categories (if given))
#  This gives a unique identification also for data files which contain data
#  on several measurands. 

#  The suffix for output referring to a stratum contains of a stratum 
#  description (e.g. F_18-29) for females, 18-29 years plus a further
#  description of the contents (e.g. F100.010 for plot 010 from segment 100).

#  The suffix for output referring to a file is a further
#  description of the contents (e.g. F095.010 for plot 010 from segment 095).

#  The suffix for output referring to a scenario describes some conditions
#  of the simulation (e.g. -Z-D2-w 0.0 summary, distance 2, w.fact=0).

#  Data file names are defined:
#  outname  (the name basis)  - in seg050
#  outname files for strata   - in seg110
#  outname files for files    - in seg080
#  outname files for scenario - in seg211

#  Plot file names are defined

#  TODO       


#  #  CHANGE HISTORY
#  17.04.2021 'stab' reduced to information that is replicate-specific.
#             General settings for the analysis are store din 'settings' / 
#             settings.csv
#  10.04.2021 Warnings concerning inconsistent rounding and drift moved to
#             beginning of final output (prior to Table 1)
#  10.12.2020 Names in stab adjusted
#  09.11.2020 Switch to new version
#  30.10.2020 Safety action if DataInfoFile has not all variables associated 
#             with data generation 
#  14.07.2020 OH category no more added to xlabel
#  21.04.2020 Loading of libraries moved to seg015 to allow their use everywhere
#  06.04.2020 New replacement strategy added to PND.ini()
#  31.03.2020 New version of PND.ini() (faster)
#  04.01.2020 generating parameters for xl added
#  03.01.2020 Method 'emp' renamed to 'npa' (nonparametric)
#  02.01.2020 Adapted to changes in methods. Only methods emp and tmc accepted.
#             log messages go to ../log/logfile  
#  27.12.2019 Sex codes are generally changed to F, M
#  13.12.2019 code, iter added to gtab
#  10.12.2019 sol.type (solution type) added to gtab
#  27.07.2019 New table 3 added (old 3,4 -> 4,5)
#  26.07.2019 Plots of RL vs age regardless of errcode
#  24.07.2019 Naming convention changed (more systematically)
#  19.07.2019 If present, OH category added to xlabel
#             Default values for non-mandatory variable set here
#  18.07.2019 Organisation of user related information simplified
#  13.07.2019 Data file information file changed to English names
#  13.07.2019 Inclusion of segments moved from start to here, where possible
#  27.05.2019 segment list updated     
#  08.05.2019 y plot limits only from estimates without error
#  11.03.2019 Start from WWW17_seg25_master  (directory ProgWW17_RH)

# ===========================================================================
if (print.log.message) { cat("%%%   TMC_seg50_master  Start\n") }
# ===========================================================================

#  Process parameter settings
source("TMC_seg040_ProcessSettings.R")  # Process parameters that may have 
                                        # been changed by the user

#  ..........................................................................
#  Clear console
#cls()

#  ..........................................................................
#  Read details of data file from info file
info <- read.table(file=info.file,sep=";",dec=".",
                   header=TRUE,stringsAsFactors=FALSE)

info.names <- colnames(info)

#  Full list of columns in the data file information
# [1] "ScenarioNo"      "FileNo"          "FileName"        "Path"           
# [5] "Source"          "DateReceived"    "Label"           "Value"          
# [9] "Rounding"        "Sex"             "SexCodeM"        "SexCodeF"       
#[13] "Age"             "DateTime"        "OH"              "Device"         
#[17] "DecChar"         "Group"           "lambda.gen"      "xc.RL1.gen"     
#[21] "xc.RL2.gen"      "xc.mode.gen"     "yc.mode.gen"     "yc.sig.gen"     
#[25] "xl.RL1.gen"      "xl.RL2.gen"      "xl.prev.gen"     "xr.RL1.gen"     
#[29] "xr.RL2.gen"      "xr.prev.gen"     "ntotal"          "x.unaffected.lo"
#[33] "x.unaffected.hi" "xc.unaffected.n" "xc.unaffected.p" "x.unaffected.p" 
#[37] "FP.change"   

#  Check for presence of mandatory columns
go.on <- TRUE

if (!("FileNo" %in% info.names))
{ cat("+++ Column 'FileNo' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("FileName" %in% info.names))
{ cat("+++ Column 'FileName' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("Path" %in% info.names))
{ cat("+++ Column  'Path'  missing in ",info.file,"  - Programme stops +++\n")
    go.on <- FALSE
} 

if (!("Value" %in% info.names))
{ cat("+++ Column 'Value' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("Rounding" %in% info.names))
{ cat("+++ Column 'Rounding' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("Sex" %in% info.names))
{ cat("+++ Column 'Sex' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("SexCodeF" %in% info.names))
{ cat("+++ Column 'SexCodeF' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("SexCodeM" %in% info.names))
{ cat("+++ Column 'SexCodeM' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

if (!("Age" %in% info.names))
{ cat("+++ Column 'Sex' missing in ",info.file,"  - programme stops +++\n")
  go.on <- FALSE
} 

#  ..........................................................................
#  Does the requested file number exist in the data information file?
#  Index of the data file to analyse in the data information file 
idx <- which(info[ ,"FileNo"] == FileNo)

if ( length(idx) == 0 )
{ cat("\n\n ++++++  Requested file no FileNo =",FileNo,
        "\n         not in                   ",info.file,
        "\n         ==> Programme stopped \n\n")
  stop("++++++  Programme stopped because file information missing  ++++")
}

if ( length(idx) > 1  )
{ cat("\n\n ++++++  Requested file no FileNo   =",FileNo,
        "\n         appears more than once in  ",info.file,
        "\n         ==> Programme stopped \n\n")
  stop("++++++  Programme stopped because of ambiguous file information ++++")
}

#  ..........................................................................

#  Interpret user input 

if (!go.on) 
{ 
  stop("+++  Programme stopped because of incomplete file information +++")
}

#  Collect information for reading the data

datafile    <- info[idx,"FileName"]

#  Die Datei mit Dateninformationen enthlt fr echte Daten den vollen 
#  Dateinamen einschl. ".csv" am Ende. Suffix abtrennen, wenn vorhanden
datafile.nchar <- nchar(datafile)
datafile.suff  <- substr(datafile,datafile.nchar-3,datafile.nchar)
if (datafile.suff == ".csv") 
{ 
  datafile <- substr(datafile,1,datafile.nchar-4)
}
datafile0   <- datafile

#  datafile and datafile0 are used below to construct the file name to read,
#  thereby accounting for the construction of names when running a 
#  simulation.

inpath      <- info[idx, "Path"]
xlabel      <- info[idx, "Label"]  # will be extended below by OH category,
                                   # if spalte.OH and use.oh are given
spalte.w    <- as.numeric(info[idx,"Value"])

#  If no label is given, use value column as label 
if (xlabel == "") { xlabel <- paste("Column", spalte.w) }

spalte.s    <- as.numeric(info[idx,"Sex"])
spalte.a    <- as.numeric(info[idx,"Age"])

dec         <- info[idx,"DecChar"]

#  round unit in info.file refers to the rounding in the original scale
#  For further use, multiply by scale.fact
round.unit  <- info[idx,"Rounding"]
round.unit  <- as.numeric(round.unit)
round.unit  <- round.unit * scale.fact

#  Columns of non-mandatory variables

if ("OH" %in% info.names)
{ spalte.oh   <- as.numeric(info[idx,"OH"]) 

  #  Complete xlabel by the OH category, if present 
  #  (OH = outpatient/hospitalized)
  #  no - not for publication
  #if (is.numeric(spalte.oh) & !is.na(use.oh)) 
  #{ xlabel <- paste(xlabel, "-", use.oh, sep="") 
  #} 
} else
{ spalte.oh   <- NA }

if ("Device" %in% info.names)
{ spalte.dev   <- as.numeric(info[idx,"Device"]) } else
{ spalte.dev   <- NA }

if ("DateTime" %in% info.names)
{ spalte.dz   <- as.numeric(info[idx,"DateTime"]) } else
{ spalte.dz   <- NA }

#  'Group' contains group membership, typically (but not exclusive)
#  available for generated data

if ("Group" %in% info.names)
{ spalte.g   <- as.numeric(info[idx,"Group"]) } else
{ spalte.g   <- NA }

#  Parameters of data generation. It is assumed that if generated data is used,
#  all parameters below are given in the file information

if ("lambda.gen" %in% info.names)
{ 
  lambda.gen  <- info[idx,"lambda.gen"]
  xc.RL1.gen  <- info[idx,"xc.RL1.gen"]
  xc.RL2.gen  <- info[idx,"xc.RL2.gen"]
  xc.mode.gen <- info[idx,"xc.mode.gen"]
  yc.mode.gen <- info[idx,"yc.mode.gen"]
  yc.sig.gen  <- info[idx,"yc.sig.gen"]
  xr.RL1.gen  <- info[idx,"xr.RL1.gen"]
  xr.RL2.gen  <- info[idx,"xr.RL2.gen"]
  xr.prev.gen <- info[idx,"xr.prev.gen"]
  xl.RL1.gen  <- info[idx,"xl.RL1.gen"]
  xl.RL2.gen  <- info[idx,"xl.RL2.gen"]
  xl.prev.gen <- info[idx,"xl.prev.gen"]
  x.unaffected.lo <- info[idx,"x.unaffected.lo"]
  x.unaffected.hi <- info[idx,"x.unaffected.hi"]

  #  Safety action, if DataFileInfo has not all variables associated with 
  #  data generation

  if (is.null(xc.RL1.gen))      { xc.RL1.gen <- NA }  
  if (is.null(xc.RL2.gen))      { xc.RL2.gen <- NA } 
  if (is.null(xc.mode.gen))     { xc.mode.gen <- NA }
  if (is.null(yc.mode.gen))     { yc.mode.gen <- NA }
  if (is.null(yc.sig.gen))      { yc.sig.gen <- NA } 
  if (is.null(xr.RL1.gen))      { xr.RL1.gen <- NA } 
  if (is.null(xr.RL2.gen))      { xr.RL2.gen <- NA } 
  if (is.null(xr.prev.gen))     { xr.prev.gen <- NA }
  if (is.null(xl.RL1.gen))      { xl.RL1.gen <- NA } 
  if (is.null(xl.RL2.gen))      { xl.RL2.gen <- NA } 
  if (is.null(xl.prev.gen))     {xl.prev.gen  <- NA } 
  if (is.null(x.unaffected.lo)) { x.unaffected.lo <- NA } 
  if (is.null(x.unaffected.hi)) { x.unaffected.hi <- NA } 

  # Action if no right or no left pathological data
  if (is.null(xl.prev.gen)) { xl.prev.gen <- 0 }
  if (is.null(xr.prev.gen)) { xr.prev.gen <- 0 }

} else
{ 
  lambda.gen  <- NA
  xc.RL1.gen  <- NA
  xc.RL2.gen  <- NA
  xc.mode.gen <- NA
  yc.mode.gen <- NA
  yc.sig.gen  <- NA
  xr.RL1.gen  <- NA
  xr.RL2.gen  <- NA
  xr.prev.gen <- NA
  xl.RL1.gen  <- NA
  xl.RL2.gen  <- NA
  xl.prev.gen <- NA
  x.unaffected.lo <- NA
  x.unaffected.hi <- NA
}

# ...........................................................................
# If the data is not generated data, then there is no information on
# the *.gen parameters. Remove "npa" from the list of methods to execute
# or display.

if (is.na(spalte.g))
{ 
  #  "npa" in meth.list?
  npa.idx <- which(meth.list=="npa")
  if (length(npa.idx) > 0)
  { # Is present - remove it
    meth.list <- meth.list[-npa.idx]
    meth.list.n  <- length(meth.list)
    #cat("\n [seg050_Master]  'npa' removed from meth.list",
    #    "\n                  Remaining methods to process:",
    #    "\n")
    #print(meth.list)
  }

  #  "npa" in gtab.list?
  npa.idx <- which(gtab.list=="npa")
  if (length(npa.idx) > 0)
  { # Is present - remove it
    gtab.list <- gtab.list[-npa.idx]
    gtab.list.n  <- length(gtab.list)
    #cat("\n [seg050_Master]  'npa' removed from gtab.list",
    #    "\n                  Remaining methods to display in gtab:",
    #    "\n")
    #print(gtab.list)
  }
} 

if (gtab.list.n == 0 | meth.list.n == 0)
{
  cat("\n ++++++ No methods to execute:\n")
  print(gtab.list)
  print(meth.list)

  go.on = FALSE

  stop("+++  Stop because of missing entries in gtab.list / meth.list +++ ")
}

# ...........................................................................

if (eval.rep)
{ #  Evaluate test data (> 1 replicates): no separation by sex
  use.sex <- NA
} else
{ #  Evaluate real data (1 replicate): separate by sex
  #  Eigenmchtigkeit von Excel beheben: Eintrag "F" wird zu FALSE
  if (!is.na(info[idx,"SexCodeF"]) &&
      info[idx,"SexCodeF"] == "FALSE") {info[idx,"SexCodeF"] <- "F" }

  use.sex <- c(info[idx,"SexCodeF"], info[idx,"SexCodeM"])
}

# =================================================================
#  Widersprche in der Eingabe beheben
#  Wenn x.lo.limit und break.min.user angegeben: 
#  Es muss gelten     break.min.user <= x.lo.limit 
if ( (!is.na(x.lo.limit)) & (!is.na(break.min.user)) &  
     (break.min.user > x.lo.limit) )
{
  break.min.user <- x.lo.limit
  cat("\n+++  Intervallstart fr Histogramme auf ",break.min.user,
      " korrigiert\n")
}
#  Weiterer Widerspruch: wenn x.lo.limit  nicht angegeben,  
#  break.min.user angegeben und break.min.user < x.min: Korrektur 
#  erst nach dem Lesen der Daten mglich, siehe dort

#  Fr die Dimensionierung interner Tabellen

#  sex  ................................................................
#  NA in use.sex sind sinnlos, rauswerfen
use.sex[use.sex == ""] <- NA
use.sex      <- use.sex[!is.na(use.sex)]
use.sex.n    <- length(use.sex)

#  Wenn eval.rep, dann nicht nach sex trennen
if (eval.rep) { use.sex.n <- 0 }

if (use.sex.n > 0)
{ # Mindestens ein echter Code - nehmen
  sex.val   <- use.sex
  sex.val.n <- length(sex.val)
} else
{ # Kein Code angegeben oder Auswertung von Replikationen -  alles nehmen 
  #  @@@ This piece of code is overruled in seg070_ReadData
  sex.val   <- "All"
  sex.val.n <- 1
} 

#  age  ................................................................
if (!is.na(spalte.a))
{ #  Alter ist angegeben - mindestens 2 Angaben ntig 
  age.limits.n <- length(age.limits)
  if (age.limits.n < 2 )
  { age.limits   <- c(age.limits, NA)
    age.limits.n <- length(age.limits)
    cat("++++  Nur 1 Angabe fr age.limits - erweitert auf \n")
    print(age.limits)
  }
  age.class.n  <- age.limits.n - 1
}  else
{ #  Alter ist nicht angegeben - age.limits intern setzen
  age.limits   <- c(NA,NA)
  age.limits.n <- length(age.limits)
  age.class.n  <- 1
}

#  Wenn eval.rep, dann keine Altersunterscheidung
if (eval.rep)
{ #  age.limits intern setzen
  age.limits   <- c(NA,NA)
  age.limits.n <- length(age.limits)
  age.class.n  <- 1
}

#  outpatient / hospitalized ..............................................
#  NULL in use.oh ist sinnlos, rauswerfen

use.oh[use.oh == ""] <- NA
use.oh      <- use.oh[!is.na(use.oh)]
use.oh.n    <- length(use.oh)
if (use.oh.n == 0) 
{ 
  use.oh   <- NA
  oh.val.n <- 1 
}

if (use.oh.n > 0)
{ # Mindestens ein echter Code - nehmen
  oh.val   <- use.oh
  oh.val.n <- length(oh.val)
} else
{ # Kein Code angegeben -  alles nehmen 
  oh.val   <- "All"
  oh.val.n <- 1
} 

#  device ..................................................................
#  NULL in use.dev ist sinnlos, rauswerfen
use.dev[use.dev == ""] <- NA
use.dev      <- use.dev[!is.na(use.dev)]
use.dev.n    <- length(use.dev)

if (use.dev.n == 0) 
{ 
  use.dev   <- NA
  oh.dev.n  <- 1 
}

if (use.dev.n > 0)
{ # Mindestens ein echter Code - nehmen
  dev.val   <- use.dev
  dev.val.n <- length(dev.val)
} else
{ # Kein Code angegeben -  alles nehmen 
  dev.val   <- "All"
  dev.val.n <- 1
} 

#  ---------------------------------------------------------------
#  Schleife ber Replikationen vorbereiten

if (!eval.rep) 
{ 
  #  Keine Replikationen auswerten 
  r.start <- 1
  r.ende  <- 1
  nrep    <- 1 
} 

#  -----------------------------------------------------------------
#  Create output tables

#  tab.s Contains all results (for all methods) for one strata (age/sex 
#        combinations) of a dataset.
#        Is overwritten by the analysis of the next stratum, if more than
#        on replicate (nrep > 1) is analysed.
#        Is created in seg100_Analysis.
#  
#  Future tab.f (still gtab) combines all tab.s for one file

#  Ausgabetabelle fr Ergebnisse der Auswertung von Replikationen 
#  anlegen
#  stab enthlt komplette Ergebnisse fr die Auswertung aller Replikationen
#  Nur, wenn dieses ein Lauf ber Replikationen von simulierten Daten ist
#  Dann: keine Auswertung nach Sex und Alter getrennt, d.h. nur 1 Ergebnis

tab.names <- c("irep", "Sex","sexlabel","Sex.num","Age","agelabel","Age.num",
               "Age.mea", "method",              
               "subset.type","n", "pct.lt.DL", "x.tr.n", "prop",
               "n.per.bin.min",
               "n.per.bin.min.eff", "n.per.bin.max.eff", "bins.n",  
               "x.tr.bin.n", 
               "x.tr.lo", "x.tr.hi", "x.unaffected.lo", "x.unaffected.hi",
               "xl.max", "xr.min",           
               "l.fact", "w.fact", "p.fact",
               "rc", "iter", "errcode",
               "lambda","mue","sigma",
               "RL1","RL2","RL1.CI","RL2.CI",
               "xc.mode","xc.mean","xc.sigma","xc.P50",
               "prev.l", "prev.c", "prev.r", "n.l", "n.c", "n.r",
               "FPR1", "FPR2", "FPR.sum",
               "opt.crit", "p.fit","chi2.total" ,"chi2.total.df",
               "chi2.trun", "chi2.trun.df",  "chi2.trun.p", "chi2.path", 
               "prev.l.tmc.pen", "prev.r.tmc.pen", "delta.RL1", "delta.RL2",
               "RL1.cilo","RL1.cihi","RL2.cilo","RL2.cihi",
               "RL1.bs.mea", "RL2.bs.mea",
               "RL1.tilo","RL1.tihi","RL2.tilo","RL2.tihi")

if (nrep > 1)
{ 
  #  Namensmenge in stab muss Teilmenge von tab.names sein (??)
  #  Bei Auswertung von Replikationen wird nur der gesamte Datensatz
  #   ausgewertet, keine Trennung nach Kategorien von 
  #  Sex oder Alter
  #  Change 17.04.2021: 
  #  'stab' contains only information that is replicate-specific. 
  #  Settings per file (and automatically also per scenario) are stored in
  #  settings / settings.csv
  
  stab.names <- c("irep", "n", "pct.lt.DL", "method", 
                  "n.per.bin.min", "n.per.bin.min.eff",
                  "rc", "iter", "errcode",
                  "prop", "x.tr.lo", "x.tr.hi",
                  "lambda","mue","sigma","p.fit","RL1","RL2",
                  "prev.l", "prev.c", "prev.r",
                  "FPR1", "FPR2", "FPR.sum")

  stab       <- matrix(NA,nrow=meth.list.n*nrep,
                       ncol=length(stab.names))
  colnames(stab)  <- stab.names
  stab            <- data.frame(stab)
  istab           <- 1              
}

#  ==========================================================================
#  Input is checked for formal errors

#  Process plot requests (plot level is defined in 060)
source("TMC_seg045_PlotRequest.R")

#  ==========================================================================
#  Show programme versions and all variables defined so far

source("TMC_seg042_SettingsSummary.R")  

#  ==========================================================================
#  Loop over replicates 

oh.sym <- "All"
if (!is.na(use.oh)) { oh.sym <- use.oh }

dev.sym <- "All"
if (!is.na(use.dev)) { dev.sym <- use.dev }

for (irep in r.start:r.ende)
{
  #  Construct input data file name.
  #  For real data (not simulated) only add the sufffix, otherwise
  #  add replicate number and suffix

  if (!eval.rep)
  { #  Names for single-file analysis   
    infile   <- paste(inpath,datafile0,".csv",sep="")
    
    #  Construct outname, the basic component for all output files
    outname <- paste(datafile0,
                     "_c",spalte.w,
                     "_o", oh.sym, 
                     "_d", dev.sym,    sep="")

  } else
  { #  Names for file sequence analysis
    #  This is the analysis of (simulated) test data. Add 3-digit replicate
    #  number to the base name.
    #  irep == 0: reserved for expected values of the data, no random error
    datafile <- paste(datafile0,formatC(irep,flag="0",width=3),sep="") 
    infile   <- paste(inpath,datafile,".csv",sep="")

    #  Construct outname, the basic component for all output files
    outname <- paste(datafile,
                     "_c",spalte.w,
                     "_o", oh.sym, 
                     "_d", dev.sym,    sep="")
    
    #  Construct more names for output files
    source("TMC_seg211_NamesPerScena.R")
  }

  #  ...............................................................
  #  If first replicate: 
  #  - define file names
  #  - remove all output files beginning with outname
  #  - organize graph windows

  if (irep == 1)
  { 

    #  Clean up
    source("TMC_seg055_RemoveOldFiles.R")
  }

  #  ...............................................................
  #  Read input data

  cat("\n\n",  
      "\n############################################################################",
      "\n#  Start of analysis for data file                                         #",
      "\n   ",infile,
      "\n############################################################################",
      "\n\n")
  
  source("TMC_seg070_ReadData.R")

  #  Now available:
  #  Input file name 
  #  Input data (in dataset)
  #  Various control parameters
  
  #  Set up plot windows for plots on file level

  source("TMC_seg046_OpenWindows.R")

  #  ...............................................................
  #  Fr die Altersangabe:
  # NA in die aktuellen min und max des Alters umwandeln

  if (is.na(age.limits[1])) 
  {   
    age.limits[1] <- 0
  }

  if (is.na(age.limits[age.limits.n])) 
  { 
    age.limits[age.limits.n] <- 999
  }

  #  -----------------------------------------------------------------
  #  Globale Ausgabetabelle fr die Auswertungen eines Datensatzes
  #  (alle Ausprgungen von Sex und Alter) anlegen
  #  Enthlt komplette Ergebnisse fr die Auswertung *eines* Datensatzes

  gtab      <- matrix(NA,nrow=meth.list.n*
                            (age.class.n*use.sex.n + 
                             age.class.n + 
                             use.sex.n + 1),
                     ncol=length(tab.names))
  colnames(gtab)  <- tab.names
  gtab            <- data.frame(gtab)

  #  -----------------------------------------------------------------
  #  Auswertung 1: ganzer Datensatz (mit Ausnahme der Bereiche, die global
  #  weggefiltert wurden)

  igtab           <- 0       # Index for rows in gtab
  iblock          <- 0       # Index for row blocks in gtab

  verlaufsplot    <- FALSE   # Ist folgende Auswertung Teil einer Auswertung 
                             # ber Kategorien von Sex oder Alter?

  #  x, age, sex, grp fr weitere Verarbeitung aus dem data frame ziehen

  x   <- dataset[ ,spalte.w]
  if (!is.na(spalte.a)) { age <- dataset[ ,spalte.a] }
  if (!is.na(spalte.s)) { sex <- toupper(dataset[ ,spalte.s]) }
  if (!is.na(spalte.g)) { grp <- toupper(dataset[ ,spalte.g]) }

  sexlabel <- sex.val[1]
  isex     <- 1
  while(isex < use.sex.n)
  { isex <- isex + 1
    sexlabel   <- paste(sexlabel,use.sex[isex],sep="+")
  }

  age.min      <- age.limits[1]
  age.max      <- age.limits[age.limits.n]
  agelabel     <- paste(age.min,"-",age.max,sep="")
  iage         <- NA

  #  Darstellung der vollstndigen Verteilung (alle Alter), getrennt nach Sex,
  #  nicht bei Simulationen
  if (!eval.rep)
  {
    sink(file=outname.file.txt,split=TRUE, append=TRUE)
    if (plot.fig090.010 & plot.fig090.020)
    { 
      source("TMC_seg090_Plot_Age_Val.R")
    }
    sink()

    #  Darstellung der Werteverteilung ber Wochentage und Tageszeit
    sink(file=outname.file.txt,split=TRUE, append=TRUE)
    if (plot.fig095.010 & plot.fig095.020)
    { 
      source("TMC_seg095_Plot_DateTime_Val.R") 
    }
    sink()
  }

  #  Vollstndige Auswertung, ganzer Datensatz
  subset.type <- 1

  if (print.log.message) { cat("%%%   TMC_seg50_master  100\n") }
  
  source("TMC_seg100_Analysis.R")

  if (print.log.message) { cat("%%%   TMC_seg50_master  110\n") }
  # ....................................................................

  #  Gesamtdaten bleiben im data frame "dataset" 
  #  Teildatenstze via Sex und Altersklasse definieren,
  #  per Indikatorvektor die betreffenden Stze aus dataset extrahieren,
  #  dabei bisheriges x und y berschreiben

  #  Wenn Sex angegeben, Trennung nach Sex
  if (!is.na(spalte.s))
  { #  Wenn > 1  Geschlechter, alle Altersklassen zusammen pro Sex
    agelabel  <- paste(age.min,"-",age.max,sep="")

    if (sex.val.n > 1)
    { for (isex in 1:sex.val.n)
      { subset <- dataset[ ,spalte.s] == sex.val[isex]
        sexlabel <- sex.val[isex]
        x   <- dataset[subset,spalte.w]
        x.n <- length(x)
        #y   <- dataset[subset,spalte.y]
        if (!is.na(spalte.a)) { age <- dataset[subset,spalte.a] }
        if (!is.na(spalte.s)) { sex <- toupper(dataset[subset,spalte.s]) }
        if (!is.na(spalte.g)) { grp <- toupper(dataset[subset,spalte.g]) }

        # Auswertung von x, Sex = Ausprgung isex
        subset.type <- 2
        source("TMC_seg100_Analysis.R")
      }
    }   #  wenn nur eine Ausprgung von sex, dann keine eigene Auswertung
  }

  #  Wenn Alter angegeben, Trennung nach Alter 

  if (!is.na(spalte.a))
  { # Altersklassen bilden 

    if (age.class.n > 1)
    { age.class <- rep(NA,times=age.class.n)
      for (iage in 2:age.limits.n)
      { #  Altersgruppen unterhalb der letzten
        age.class[iage-1] <- paste(age.limits[iage-1],"-",
                                   age.limits[iage]-1,sep="")
        #  Letzte Altergruppe
        if (iage == age.limits.n)
        { age.class[iage-1] <- paste(age.limits[iage-1],"-",
                                     age.limits[iage],sep="")
        }
      }
    } else
    { # keine Altersklassen
      age.class <- paste("Alle")
    } 
 
    #  Wenn > 1  Altersklassen, jede Klasse fr alle Ausprgungen von Sex
    if (age.class.n > 1)
    { sexlabel <- use.sex[1]
      isex <- 1
      while(isex < use.sex.n)
      { isex <- isex + 1
        sexlabel   <- paste(sexlabel,use.sex[isex],sep="+")
      }
      for (iage in 1:age.class.n)
      { if (iage < age.class.n) 
        { subset <- (age.limits[iage]    <= dataset[ ,spalte.a]) & 
                    (dataset[ ,spalte.a] <  age.limits[iage+1])
        } else
        { subset <- (age.limits[iage]    <= dataset[ ,spalte.a]) & 
                    (dataset[ ,spalte.a] <=  age.limits[iage+1])
        }

        agelabel <- age.class[iage]
        x   <- dataset[subset,spalte.w]
        x.n <- length(x)
        #y   <- dataset[subset,spalte.y]
        if (!is.na(spalte.a)) { age <- dataset[subset,spalte.a] }
        if (!is.na(spalte.s)) { sex <- toupper(dataset[subset,spalte.s]) }
        if (!is.na(spalte.g)) { grp <- toupper(dataset[subset,spalte.g]) }

        # Auswertung von x, Alter = Ausprgung iage
        verlaufsplot <- TRUE
        subset.type  <- 3
        source("TMC_seg100_Analysis.R")

      } # iage in 1:age.class.n
    }   #  age.class.n > 1  
  }     #  

  #  Wenn Sex und Alter angegeben, Trennung nach Sex und Altersklasse
  if (sex.val.n > 1 &  age.class.n > 1)
  { for (isex in 1:sex.val.n)
    { subset.s <- dataset[ ,spalte.s] == sex.val[isex]
      sexlabel <- sex.val[isex]

      for (iage in 1:age.class.n)
      { if (iage < age.class.n) 
        { subset <- subset.s &
                    (age.limits[iage]    <= dataset[ ,spalte.a]) & 
                    (dataset[ ,spalte.a] <  age.limits[iage+1])
        } else
        { subset <- subset.s &
                    (age.limits[iage]    <= dataset[ ,spalte.a]) & 
                    (dataset[ ,spalte.a] <=  age.limits[iage+1])
        }

        agelabel <- age.class[iage]
        x   <- dataset[subset,spalte.w]
        x.n <- length(x)
        #y   <- dataset[subset,spalte.y]
        if (!is.na(spalte.a)) { age <- dataset[subset,spalte.a] }
        if (!is.na(spalte.s)) { sex <- toupper(dataset[subset,spalte.s]) }
        if (!is.na(spalte.g)) { grp <- toupper(dataset[subset,spalte.g]) }

        # Auswertung von x, Sex = Ausprgung isex und Alter = iage
        verlaufsplot   <- TRUE
        subset.type    <- 4
        source("TMC_seg100_Analysis.R")
      }   #  iage in ...
    }     #  isex in ...
  }       #  !is.na(spalte.a) ...

  # ---------------------------------------------------------------------------
  #  Output for a complete file, all Sex*Age combinations

  if (print.log.message) { cat("%%%   TMC_seg50_master  200\n") }

  # ...........................................................................
  #  Overview

  sink(file=outname.file.txt,split=TRUE, append=FALSE)

  cat("\n\n===========================================================================",
      "\n\nEstimation of Reference Limits ",
      "\nSUMMARY OF RESULTS FOR ALL DATA SUBSETS (STRATA)",
      "\nIdentification of this run             ", RunId,
      "\nComplete list of parameters in         ", logfile,
      "\n",
      "\nUser                                   ", user,
      "\nData file information in               ", info.file,
      "\nStart of this run                      ", format(time.start),
      "\nData file                              ",
      "\n   ",infile,
      "\nResults in file                        ",
      "\n   ",outname.stra.txt,
      "\nFilter  outpatient/hospitalized        ", use.oh,
      "\nFilter  device                         ", use.dev,
      "\nFilter  analysis time of day           ", paste(ana.hour.min, ":",
                                                         ana.minu.min, " - ",
                                                         ana.hour.max, ":",
                                                         ana.minu.max, sep=""),
      "\nInput values multiplied by             ", scale.fact,
      "\nScaled values are limited by           ", x.lo.limit,x.hi.limit,
      "\nScaled values are rounded here to      ",round.unit,
      "\nSurrogate factor for values < DL       ", s.fact, 
      "\nTotal # of values in data file         ",x.n1,
      "\n# of values after filtering, sampling  ",x.n,
      "\n",
      "\nSmooth empirical histogram?            ",smooth.hist1, smooth.hist2,
      "\nMin. # of values per bin               ",n.per.bin.min,
      "\nMin. # of bins in truncation interval  ",x.tr.bins.min,     
      "\nMin. # of values in a stratum          ",x.n.min,
      "\nUser-specified min. prop. in TI        ", x.tr.prop.min, 
      "\nUser-specified max. prop. in TI        ", x.tr.prop.max,     
      "\nWeight of truncation interval length   ",l.fact,
      "\nPenalty factor 'wrong prevalence'      ",p.fact,
      "\nPenalty factor 'wrong prediction'      ",w.fact,
      "\nMinimal lambda                         ",lambda.min,
      "\nMaximal lambda                         ",lambda.max,
      "\n",
      "\nProbabilities < RL1 / < RL2            ",RL1.p,"/ ",RL2.p,   
      "\n===========================================================================",
      "\n\n")

  # ...........................................................................
  #  Before printing detailed result tables: repeat warnings

  if (print.log.message) { cat("%%%   TMC_seg50_master  220\n") }

  # ...........................................................................
  #  Consistent rounding?
  #  Calculations were made in seg070

  cat("\n", rep("=", times=76), "\n", sep="")
  cat("\n Check for inconsistent rounding in the input data\n")

  if (ldt.p < 0.05)
  {
    cat("\n", rep("+", times=76), "\n", sep="")
    cat("\n p value for H0: 'Last digits are equally likely': ", ldt.p,"\n")
    cat("\n +++ Consistent rounding for the whole data set is questionable +++\n")
    cat("\n    Frequencies of the last digit", 
           "('Observed' should be close to 'Expected') \n\n")
    print(ldt)
    cat("\n", rep("+", times=76), "\n", sep="")
  }  else      
  {
    cat("\n p value for H0: 'Last digits are equally likely': ", ldt.p,"\n")
    cat("\n *** Rounding seems consistent ***\n")
  }
  cat("\n", rep("=", times=76), "\n\n", sep="")

  if (print.log.message) { cat("%%%   TMC_seg50_master  230\n") }

  # ...........................................................................
  #  Drift over time? 
  #  Only if requested and if date of measurement is provided. Then 'dataset'
  #  contains ana.mon, ana.yea
  if (plot.fig050.010 && 
      (("ana.mon" %in% colnames(dataset)) & ("ana.yea" %in% colnames(dataset))))
  { 
    dev.set(fig050.010)
    par(mfrow=c(2,1))

    #  ana.mon, ana.yea are present. ana.day not needed as we are looking for
    #  monthly medians

    dataset <- data.frame(dataset, 
                          DM=as.Date(paste("15.", dataset[ , "ana.mon"], ".", 
                                                  dataset[ , "ana.yea"],
                                           sep=""), format = "%d.%m.%Y") )
    dataset[1:10, ]
    as.numeric(dataset[1:10, "DM"])

    dataset <- dataset[order(dataset[ ,"DM"]), ]

    #  Check drift for F
    ok <- dataset[ , spalte.s] == "F"
    subt <- paste(RunId, " / " ,outname, "_F / n=", sum(ok), sep="")

    #  Take empirical quantiles of all data as RLs. No age stratification.
    RL1.F.emp <- Quantile(dataset[ok, spalte.w], probs=RL1.p)
    RL2.F.emp <- Quantile(dataset[ok, spalte.w], probs=RL2.p)

    drift.F <- Drift(dataset[ok, "DM"], dataset[ok, spalte.w], 
                     RL1.F.emp, RL2.F.emp, fig050.010, xlabel, 
                     "Drift for sex=F",
                     subt=subt, gamcol=blue.32)

    ok <- dataset[ , spalte.s] == "M"
    subt <- paste(RunId, " / " ,outname, "_M / n=", sum(ok), sep="")

    #  Take empirical quantiles of all data as RLs. No age stratification.
    RL1.M.emp <- Quantile(dataset[ok, spalte.w], probs=RL1.p)
    RL2.M.emp <- Quantile(dataset[ok, spalte.w], probs=RL2.p)

    drift.M <- Drift(dataset[ ,"DM"], dataset[ , spalte.w], 
                     RL1.M.emp, RL2.M.emp, fig050.010, xlabel, 
                     "Drift for sex=M",
                     subt=subt, gamcol=blue.32)

    cat("\n", rep("=", times=76), "\n", sep="")

    cat("\n Check for drift of values over time\n")
    cat("\n The blue confidence band should overlap",
            "the gray range at all times\n\n")
    cat("\n See window ", fig050.010,
        "\n and        ", fig050.010.file,
        "\n") 

    drifttext <- "No drift"
    if (drift.F) { drifttext <- "Drift detected" }
    cat("\n    Sex = F: ", drifttext) 

    drifttext <- "No drift"
    if (drift.M) { drifttext <- "Drift detected" }
    cat("\n    Sex = M: ", drifttext) 

    savePlot(file=fig050.010.file, type=figtype)

    cat("\n\n", rep("=", times=76), "\n\n", sep="")
  }
  
  if (print.log.message) { cat("%%%   TMC_seg50_master  240\n") }
 
  # ...........................................................................
  #  Detailed tables


  #  Add calculated quantities
  gtab <- data.frame(gtab, x.tr.pr=gtab[ ,"x.tr.n"]/gtab[ ,"n"])

  #  Create table for printing. Here: keep both tables, including empty rows
  gtab.print <- gtab
 
  #  Produce print table containing only non-empty rows
  gtab.print <- gtab.print[!is.na(gtab.print[ , "method"]), ] 

  #  Format some numbers for nicer printing
  #  width: size of output field
  #  digits: number of digits to show. Overrides width, if necessary.
  #  flag="#": show trailing zeroes  
  #  FC is shortcut for formatC(x, format="f", width=w, digits=d, flag="#")
  #  including correct treatment of NA:
  #  FC <- function(x, w, d)

  gtab.print[ ,"chi2.total"]    <- format(gtab[ ,"chi2.total"],digits=4)
  gtab.print[ ,"chi2.total.df"] <- format(gtab[ ,"chi2.total.df"],digits=4)
  gtab.print[ ,"chi2.trun"]     <- format(gtab[ ,"chi2.trun"],digits=4)
  gtab.print[ ,"chi2.trun.df"]  <- format(gtab[ ,"chi2.trun.df"],digits=4)
  gtab.print[ ,"chi2.path"]     <- format(gtab[ ,"chi2.path"],digits=4)
  gtab.print[ ,"opt.crit"]      <- FC(gtab[ ,"opt.crit"], w=NA, d=2) 

  gtab.print[ ,"p.fit"]         <- FC(gtab[ ,"p.fit"], w=6, d=4) 
  gtab.print[ ,"pct.lt.DL"]     <- FC(gtab[ ,"pct.lt.DL"], w=6, d=1)
  gtab.print[ ,"prev.l"]        <- FC(gtab[ ,"prev.l"], w=6, d=3)
  gtab.print[ ,"prev.c"]        <- FC(gtab[ ,"prev.c"], w=6, d=3)
  gtab.print[ ,"prev.r"]        <- FC(gtab[ ,"prev.r"], w=6, d=3)
  gtab.print[ ,"RL1"]           <- FC(gtab[ ,"RL1"],w=NA, d=3)
  gtab.print[ ,"RL2"]           <- FC(gtab[ ,"RL2"],w=NA, d=3)
  gtab.print[ ,"x.tr.pr"]       <- FC(gtab.print[ ,"x.tr.pr"], w=6, d=3)
  gtab.print[ ,"delta.RL1"]     <- FC(gtab[ ,"delta.RL1"], w=NA, d=3)
  gtab.print[ ,"delta.RL2"]     <- FC(gtab[ ,"delta.RL2"], w=NA, d=3)

  gtab.print[ ,"RL1.cilo"]      <- FC(gtab[ ,"RL1.cilo"], w=NA, d=3)
  gtab.print[ ,"RL1.cihi"]      <- FC(gtab[ ,"RL1.cihi"], w=NA, d=3)
  gtab.print[ ,"RL2.cilo"]      <- FC(gtab[ ,"RL2.cilo"], w=NA, d=3)
  gtab.print[ ,"RL2.cihi"]      <- FC(gtab[ ,"RL2.cihi"], w=NA, d=3)

  gtab.print[ ,"RL1.bs.mea"] <- FC(gtab[ ,"RL1.bs.mea"], w=NA, d=4)
  gtab.print[ ,"RL1.tilo"]   <- FC(gtab[ ,"RL1.tilo"], w=NA, d=4)
  gtab.print[ ,"RL1.tihi"]   <- FC(gtab[ ,"RL1.tihi"], w=NA, d=4)
  gtab.print[ ,"RL2.bs.mea"] <- FC(gtab[ ,"RL2.bs.mea"], w=NA, d=4)
  gtab.print[ ,"RL2.tilo"]   <- FC(gtab[ ,"RL2.tilo"], w=NA, d=4)
  gtab.print[ ,"RL2.tihi"]   <- FC(gtab[ ,"RL2.tihi"], w=NA, d=4)

  gtab.print[ ,"lambda"]   <- FC(gtab[ ,"lambda"], w=6, d=3)
  gtab.print[ ,"mue"]      <- FC(gtab[ ,"mue"], w=6, d=3)
  gtab.print[ ,"sigma"]    <- FC(gtab[ ,"sigma"], w=6, d=3)

  #  Rename col names for printing
  gtab.names       <- colnames(gtab)
  gtab.print.names <- gtab.names

  gtab.print.names[which(gtab.names=="subset.type")]    <- "sst" 

  gtab.print.names[which(gtab.names=="errcode")]    <- "err"
  gtab.print.names[which(gtab.names=="pct.lt.DL")]  <- "pct.DL"
  gtab.print.names[which(gtab.names=="method")]     <- "meth"
  gtab.print.names[which(gtab.names=="prev.l")]     <- "prevL"
  gtab.print.names[which(gtab.names=="prev.c")]     <- "prevC"
  gtab.print.names[which(gtab.names=="prev.r")]     <- "prevR"

  gtab.print.names[which(gtab.names=="x.RL1.cilo")] <- "RL1.cilo"
  gtab.print.names[which(gtab.names=="x.RL1.cihi")] <- "RL1.cihi"
  gtab.print.names[which(gtab.names=="x.RL2.cilo")] <- "RL2.cilo"
  gtab.print.names[which(gtab.names=="x.RL2.cihi")] <- "RL2.cihi"

  gtab.print.names[which(gtab.names=="chi2.total")]    <- "c2.to"
  gtab.print.names[which(gtab.names=="chi2.total.df")] <- "to.df"
  gtab.print.names[which(gtab.names=="chi2.trun")]     <- "c2.tr"
  gtab.print.names[which(gtab.names=="chi2.trun.df")]  <- "tr.df"
  gtab.print.names[which(gtab.names=="chi2.path")]     <- "c2.pa"

  gtab.print.names[which(gtab.names=="x.tr.bin.n")]        <- "xtbn"
  gtab.print.names[which(gtab.names=="n.per.bin.min.eff")] <- "nb.min" 
  gtab.print.names[which(gtab.names=="n.per.bin.max.eff")] <- "nb.max" 

  names(gtab.print)    <- gtab.print.names
  rownames(gtab.print) <- NULL
  
  # ...........................................................................

  #  Part 1
  cat("\n--------------------------------------------------------------------------",
      "\n       Table 1: Main results",
      "\n--------------------------------------------------------------------------",
      "\n")
  print(gtab.print[ ,c("Sex","Age","n", "meth",
                       "RL1","RL2", 
                       "prevL", "prevC", "prevR", "p.fit", "opt.crit", "err") ])
  cat("--------------------------------------------------------------------------",
      "\n")
 
    #  Part 2a
    cat("\n--------------------------------------------------------------------------",
      "\n       Table 2a: Asymptotic CIs for RILs",
      "\n--------------------------------------------------------------------------",
      "\n")
    print(gtab.print[ ,c("Sex","Age", "n", "meth", "pct.DL",
                 "RL1.cilo",  "RL1.cihi", "RL2.cilo",  "RL2.cihi",
                 "p.fit")])
    cat("-------------------------------------------------------------------------",
      "\n")

    if (nbs > 0)
    {
    #  Part 2b  @@@ not updated to tmc4
    cat("\n--------------------------------------------------------------------------",
      "\n       Table 2b: Bootstrap CIs for RILs (nbs =", nbs, ")",
      "\n--------------------------------------------------------------------------",
      "\n")
    print(gtab.print[ ,c("Sex","Alter", "n", "meth",
                 "RL1.bs.mea", "RL1.tilo",  "RL1.tihi", 
                 "RL2.bs.mea", "RL2.tilo",  "RL2.tihi")])
    cat("--------------------------------------------------------------------------",
      "\n")
    }

  if (print.6tables)
  {
    #  Print more details
    #  Part 3
    cat("\n--------------------------------------------------------------------------",
      "\n       Table 3: Estimated PND parameters",
      "\n--------------------------------------------------------------------------",
      "\n")
    print(gtab.print[ ,c("Sex","Age","n",  "x.tr.pr", "meth",
                 "rc", "iter", "lambda","mue","sigma") ])
    cat("--------------------------------------------------------------------------",
      "\n")

    #  Part 4
    cat("\n--------------------------------------------------------------------------",
      "\n       Table 4: Properties of the estimated PND distribution",
      "\n--------------------------------------------------------------------------",
      "\n")
    print(gtab.print[ ,c("Sex","Age", "n", "meth",
                 "xc.mode","xc.mean","sigma","xc.P50") ])
    cat("----------------------------------------------------------------------------",
      "\n")

    #  Part 5
    cat("\n--------------------------------------------------------------------------",
      "\n       Table 5: Diagnostic quantities: fit statistics",
      "\n--------------------------------------------------------------------------",
      "\n")
    print(gtab.print[ ,c("Sex","Age","n",  "meth",
                 "c2.to", "to.df", "c2.tr", "tr.df", "c2.pa",
                 "p.fit", "opt.crit") ])
    cat("----------------------------------------------------------------------------",
      "\n")

    #  Part 6

    #  If data is simulated:
    #  Produce table of relative errors
    #@@@ 02.12.2020

#@@@    if (!is.na(lambda.gen))
#@@@    { 
#@@@    delta.RL1.sum <- sum(as.numeric(gtab.print[ ,"delta.RL1"]), na.rm=TRUE)
#@@@    delta.RL2.sum <- sum(as.numeric(gtab.print[ ,"delta.RL2"]), na.rm=TRUE)
#@@@   delta.RL.sum  <- delta.RL1.sum + delta.RL2.sum 

#@@@    cat("\n-----------------------------------------------------------------------",
#@@@        "\n       Table 6: Diagnostic quantities: relative errors",
#@@@        "\n-----------------------------------------------------------------------",
#@@@        "\n")
#@@@    print(gtab.print[ ,c("Sex","Age","sst","n", "x.tr.n", "meth",
#@@@                         "counts.n", "xtbn", "nb.min", "nb.max", 
#@@@                          "delta.RL1", "delta.RL2") ])
#@@@    cat("-------------------------------------------------------------------------",
#@@@        "\n      Sum(delta.RL1)                  ", delta.RL1.sum,
#@@@        "\n                       Sum(delta.RL2) ", delta.RL2.sum,
#@@@        "\n      Sum(delta.RL1) + Sum(delta.RL2) ", delta.RL.sum,
#@@@        "\n" )
#@@@    }
  }              #  print.6tables ....

  #  Kopie von gtab (!) als csv-Datei 
  write.table(gtab,file=outname.file.csv,row.names=FALSE,col.names=TRUE,
              sep=";",dec=".")

  #  Falls dieses die Auswertung einer Replikation ist:
  #  Ergebnis dieser Analyse in stab einfllen
  if (nrep > 1)
  {
    stab[istab:(istab+meth.list.n-1), stab.names] <- gtab[ ,stab.names]
    istab <- istab + meth.list.n
  }

  # ===========================================================================
  #  If this analysis is not an analysis of several replicates (usually
  #  simulated datasets): plot estimated RLs by age, if the data allows

  #  More than  > 1 Sex and  > 1 age class?

  if (sex.val.n > 1 & age.class.n > 1 & 
      (!is.na(Verlauf.Meth1) & !is.na(Verlauf.Meth1) ))
  {
    #  Verlauf ber Altersklassen plotten, 
    #  getrennt nach Sex, 
    #  2 Methoden in ein Bild
    #  Altersangabe = Grenzen der Altersklassen

    #  Methode zum Plotten als Verlauf wurden bei den speziellen 
    #  Benutzereingaben ausgewhlt (Verlauf.Meth1, Verlauf.Meth2)
    #  Farben aus dem style file holen

    meth1col <- NA
    meth1pch <- NA
    meth2col <- NA
    meth2pch <- NA

    if (!is.na(Verlauf.Meth1))
    { meth1col <- meth.dsg[which(meth.dsg[ ,1] == Verlauf.Meth1),2] 
      meth1pch <- as.numeric(meth.dsg[which(meth.dsg[ ,1] == Verlauf.Meth1),3])
    }
    if (!is.na(Verlauf.Meth2))
    { meth2col <- meth.dsg[which(meth.dsg[ ,1] == Verlauf.Meth2),2] 
      meth2pch <- as.numeric(meth.dsg[which(meth.dsg[ ,1] == Verlauf.Meth2),3])
    }

    #  Plot scale depends only on the range of the methods to plot
  
    Verlauf.liste <- c(Verlauf.Meth1,Verlauf.Meth2)
    Verlauf.liste <- Verlauf.liste[!is.na(Verlauf.liste)]
    ok <- gtab[ ,"method"] %in% Verlauf.liste

    #  If there are user definitions for the RL axis use these, else
    #  calculate own values
    if (!is.na(RL.clip.min))
    {
      yplotmin150.010 <- RL.clip.min
      yplotmax150.010 <- RL.clip.max
      yplotby1150.010 <- RL.clip.by1
      yplotby2150.010 <- RL.clip.by2
    } else
    {
      yplotmin150.010 <- min(as.numeric(gtab[ok,"RL1.cilo"]),na.rm=TRUE)
      yplotmin150.010 <- 0
      yplotmax150.010 <- max(as.numeric(gtab[ok,"RL2.cihi"]),na.rm=TRUE)
      yplotby1150.010 <- NA
      yplotby2150.010 <- NA
    }

    dx <- 0.1       #  Versatz method 1 gegen method 2

  }   # if (sex.val.n > 1 & age.class.n > 1)

  # ----------------------------------------------------------------------
  #  Verlauf mit Glttung, Kurven fr F / M /F+M, 1 method
  if (age.class.n > 1)
  {
    #  Verlauf ber Altersklassen plotten, 
    #  getrennt nach Sex (F / M / F+M), 
    #  1 methode in ein Bild
    #  Altersangabe = Grenzen der Altersklassen
    #  Konfidenz/Toleranzintervall nach Verfgbarkeit

    #  Bilder mit reduzierter Information, F und M getrennt, 
    #  fr Verffentlichung

    #  Produce plots of RL vs age for all requested methods except npa
    #  Graph windows

    source("TMC_seg149_OpenWindows.R")

    if ("npa" %in% meth.list)
    {
      RL.age.meth <- "npa"
      PlotId      <- fig150.010
      PlotFile    <- fig150.010.file
      source("TMC_seg150_PlotRL_vs_Age_by_S.R")
    }
    if ("qqw" %in% meth.list)
    {
      RL.age.meth <- "qqw"
      PlotId      <- fig150.020
      PlotFile    <- fig150.020.file
      source("TMC_seg150_PlotRL_vs_Age_by_S.R")
    }
    if ("bha" %in% meth.list)
    {
      RL.age.meth <- "bha"
      PlotId      <- fig150.030
      PlotFile    <- fig150.030.file
      source("TMC_seg150_PlotRL_vs_Age_by_S.R")
    }
    if ("tuk" %in% meth.list)
    {
      RL.age.meth <- "tuk"
      PlotId      <- fig150.040
      PlotFile    <- fig150.040.file
      source("TMC_seg150_PlotRL_vs_Age_by_S.R")
    }
    if ("tmc" %in% meth.list)
    {
      RL.age.meth <- "tmc"
      PlotId      <- fig150.050
      PlotFile    <- fig150.050.file
      source("TMC_seg150_PlotRL_vs_Age_by_S.R")
    }
    if ("tmu" %in% meth.list)
    {
      RL.age.meth <- "tmu"
      PlotId      <- fig150.060
      PlotFile    <- fig150.060.file
      source("TMC_seg150_PlotRL_vs_Age_by_S.R")
    }

    #  Bilder mit reduzierter Information, nur F + M, 
    #  fr Verffentlichung

    source("TMC_seg160_PlotRL_vs_Age.R")

  }   # if (age.class.n > 1)

  #  ......................................................................

  cat("\n\n",  
      "\n############################################################################",
      "\n#  End of analysis for data file                                           #",
      "\n   ",infile,
      "\n############################################################################",
      "\n\n")

  #  Ausgabe der Gesamttabelle fr diese Datendatei in die Ergebnisdatei beenden
  sink()

}  #  for (irep in 1:nrep) 

# ===========================================================================
#  If this analysis *is* an analysis of several replicates (usually
#  simulated datasets): plot estimated RLs over all replicates

if (nrep > 1)
{
  #  Organize filenames and plot windows 

  write.table(stab, file=soutfile.tmc.csv, 
              row.names=FALSE, col.names=TRUE, 
              quote=FALSE, sep=";", dec=".")

  source("TMC_seg202_SimResults.R")
}

# ===========================================================================
#  End of execution

time.end   <- Sys.time()
cat("\n\nExecution time: ",format(difftime(time.end,time.start)),"\n")
cat("\n")

# ===========================================================================
if (print.log.message) { cat("%%%   TMC_seg50_master  End\n") }
# ===========================================================================
