#  TMC_seg100_Analysis.R 
#  (c) wwosniok@math.uni-bremen.de

#  Truncated minimum chi-square estimation
#
#  Segment doing the analysis per stratum

#  #  CHANGE HISTORY

#  02.05.2021 Result plots per method moved to method-specific segments
#  25.04.2021 Tukey approach added
#  03.03.2021 Basic information per stratum always in output table
#  22.02.2021 Calculation of x.tr.prop.limits moved to 040_ProcessSettings 
#  15.02.2021 Subsegments outsourced (100_Analysis_QQW, ...)
#  26.01.2021 meth.list made effective
#  13.01.2021 r.fact introduced
#  11.01.2021 Search sequence for TI changed to large -> small with 
#             intermediate checks for acceptability
#  04.01.2021 Smoothing strategy changed, applied to uncollapsed equidistant 
#             data 
#  17.12.2020 Graph windows opened by seg046
#  07.12.2002 Safety actions if QQW produces no result
#  01.12.2020 Organisation of histogram calculation, smoothing and reduction
#             changed, now done by FindHisto()
#  16.11.2020 x.kde added to call QQW()
#  14.11.2020 bins.n.min added to call QQW()
#  09.11.2020 New approach of analysis started. TMC is now a function, 
#             qqw and Bhattacharya added. New approach for kde bandwidth 
#             and histogram bandwidth.

#  21.08.2020 Save plot100.040 only for regular analysis, not simulation 
#             Unaffected range taken from DataInfoFile, not calculated here
#  18.08.2020 x.clip.by1 == NA or x.clip.by2 == NA correctly processed 
#  21.07.2020 Pr.lo, Pr.hi set to 0.001, 0.999 (previously 0.01, 0.95)
#             Setting moved to TMC_seg015_DefaultSettings.R
#  20.07.2020 Detail output for startum in txt file re-organized
#  16.07.2020 Common par parameters for plotting
#  01.07.2020 Check generated data unified
#  30.05.2020 Logarithmic x axis in fig100.010
#  25.05.2020 RB3[2, ] expanded: rule for y[1]==0 added
#     05.2020 Plotting of density with new approach for density bw added
#  11.05.2020 RB3[2, ] gendert
#  29.04.2020 Plots newly arranged. kde plot still excluded.
#  21.04.2020 x.kde calculated from only an interior range of the data
#  29.03.2020 opt.crit als einheitliche Bezeichnung
#  11.03.2020 Strategie vorbergehend zurckgesetzt, neuen Namen beibehalten
#  10.03.2020 Strategie fr Bandbreitenberechnung gendert 
#  19.02.2020 tmc.master4 replaced by tmc.master5 (hierarchical search of
#             truncation interval)
#  24.01.2020 New structure for stopping execution if not enough data
#  23.01.2020 Break values for histogram calculated by find.breaks,
#             construction keeps n.per.bin.min and x.bins.min, if possible
#  18.01.2020 QQ plot based calculation of initial values
#  05.01.2020 Plot numbers changed
#  04.01.2020 Notation unified: always a . after x, y in x.hist, x.breaks, ...
#             No x or y with x.counts: only 'counts'
#  03.01.2020 New organisation: only tmc is done, new strategy for finding
#             the optimal truncation interval, initial value selection 
#             simplified, results are stored in 'tab' only  
#  05.11.2019 Plots for bootstrapping updated
#  27.10.2019 More information for BS result in output table
#  23.10.2019 TMC estimation applied to smoothed histogram
#  12.10.2019 BS calculation of CI activated
#  10.10.2019 TMC part of the analysis moved to TMC_seg101_Analysis_TMC.R
#  28.08.2019 prop.tmc.seq calculation revised to chi2 contribution
#             p.fit.min introduced, see Expert settings
#  27.08.2019 prop.tmc.seq calculation revised
#  14.08.2019 prop.tmc.seq shifted to interval above detection limit
#  26.07.2019 In RL.est: xc.mode, xc.sig, xc.P50 added, also for
#             methods other than IFCC 
#             In RL.est: x.mode,  y.mode,  y.sig replaced by
#                        xc.mode, yc.mode, yc.sig 
#  24.07.2019 Calculation prop.qqw changed
#             New file naming convention, see seg050
#  22.07.2019 Typing error in optimality criterion removed
#  13.07.2019 Sequence for prop.tmc candidates defined in 
#             TMC_seg060_ExpertSettings.R
#  02.07.2019 names of TMC result variables corrected
#  01.07.2019 RL1.p, RL2.p  instead of fixed probabilities
#  08.06.2019 Empirische Quantile grundstzlich berechnet
#  12.05.2019 errcode fr tmc noch einmal gendert
#             prop.qqw and prop.tmc are dynamcally calculated, depending
#             on the number of values < detection limit
#  08.05.2019 errcode fr tmc gendert
#  07.05.2019 qqw operates on x with ties replaced by increasing sequence
#  05.04.2019 y.mode hier berechnet
#  12.03.2019 Start from WW17_seg40  (directory ProgWW17_RH)

# ===========================================================================

if (print.log.message) { cat("%%%   TMC_seg100_Analysis  Start\n") }

# ===========================================================================

# ......................................................................
#  Clean up. From here on processing of a new data subset.

if (exists("x.kde"))  { rm(x.kde) }
if (exists("y.kde"))  { rm(y.kde) }
if (exists("x.hist")) { rm(x.hist) }
if (exists("y.hist")) { rm(y.hist) }

# ......................................................................
#  Initialise error messages

ErrTxt   <- c("Not enough values in stratum",
              "Not enough histogram bins of requested size in stratum",
              "Requested number of bins reduced")

ErrTxt.n <- length(ErrTxt)

ErrMsg <- data.frame(ErrNum=1:ErrTxt.n,
                     Occurred=rep(FALSE, times=ErrTxt.n),
                     ErrTxt,
                     stringsAsFactors=FALSE)

#  Set back execution control. go.on == FALSE means no further execution
#  in this stratum
go.on <- TRUE

# ......................................................................
#  Define stratum specific output file names

source("TMC_seg110_NamesPerStratum.R")  

# ......................................................................
#  Graph windows

source("TMC_seg099_OpenWindows.R")

# ......................................................................
#  Create results tables
#  All results from this segment, which refer to one stratum,
#  go into 'tab.stra'. Column names are given in tab.names, defined
#  in seg050.
#  Information on data generation was read by read.table and is kept in 
#  the relevant variables - see below

#  Create output table for the actual stratum
tab.stra           <- matrix(NA,nrow=meth.list.n, ncol=length(tab.names))
dimnames(tab.stra) <- list(meth.list, tab.names)
tab.stra           <- data.frame(tab.stra)

# ......................................................................
#  Separator lines for output tables
sep.p <- str_dup(".", 78)
sep.m <- str_dup("-", 78)
sep.e <- str_dup("=", 78)

# ============================================================================
#  Decribe the actual stratum

if (print.log.message) { cat("%%%   TMC_seg100_Analysis  Position 050\n") }

x.n <- length(x)
subtitle <- paste(RunId," / ",outname.stra," / n =",x.n)

#  Wieviele Werte unter NWG?
if (!is.na(detect.limits.max))
{
  x.lt.detect.n <- sum(x <= detect.limits.max)
} else
{
  x.lt.detect.n <- 0
}
x.lt.detect.pct <- 100*x.lt.detect.n/x.n

#  @@@@@ check if  x.lt.detect.pct < allowed maximum

if (exists("age")) 
{ Age.mea <- mean(age) } else
{ Age.mea <- NA }

# ----------------------------------------------------------------------------
#  Put basic information in tab.stra, also for subsets that will not be 
#  analysed below due to too few cases

if ("bha" %in% meth.list)
{
  tab.stra["bha", "irep"]    <- irep
  tab.stra["bha", "method"]  <- "bha"

  tab.stra["bha", "sexlabel"]    <- sexlabel
  tab.stra["bha", "agelabel"]    <- agelabel
  tab.stra["bha", "Age.num"]     <- iage
  tab.stra["bha", "Age.mea"]     <- Age.mea

  tab.stra["bha","subset.type"] <- subset.type
  tab.stra["bha", "n"]          <- x.n
  tab.stra["bha", "pct.lt.DL"]  <- x.lt.detect.pct
}

if ("npa" %in% meth.list)
{
  tab.stra["npa", "irep"]    <- irep
  tab.stra["npa", "method"]  <- "npa"

  tab.stra["npa", "sexlabel"]    <- sexlabel
  tab.stra["npa", "agelabel"]    <- agelabel
  tab.stra["npa", "Age.num"]     <- iage
  tab.stra["npa", "Age.mea"]     <- Age.mea

  tab.stra["npa","subset.type"] <- subset.type
  tab.stra["npa", "n"]          <- x.n
  tab.stra["npa", "pct.lt.DL"]  <- x.lt.detect.pct
}

if ("qqw" %in% meth.list)
{
  tab.stra["qqw", "irep"]    <- irep
  tab.stra["qqw", "method"]  <- "qqw"

  tab.stra["qqw", "sexlabel"]    <- sexlabel
  tab.stra["qqw", "agelabel"]    <- agelabel
  tab.stra["qqw", "Age.num"]     <- iage
  tab.stra["qqw", "Age.mea"]     <- Age.mea

  tab.stra["qqw","subset.type"] <- subset.type
  tab.stra["qqw", "n"]          <- x.n
  tab.stra["qqw", "pct.lt.DL"]  <- x.lt.detect.pct
}

if ("tmc" %in% meth.list)
{
  tab.stra["tmc", "irep"]    <- irep
  tab.stra["tmc", "method"]  <- "tmc"

  tab.stra["tmc", "sexlabel"]    <- sexlabel
  tab.stra["tmc", "agelabel"]    <- agelabel
  tab.stra["tmc", "Age.num"]     <- iage
  tab.stra["tmc", "Age.mea"]     <- Age.mea

  tab.stra["tmc","subset.type"] <- subset.type
  tab.stra["tmc", "n"]          <- x.n
  tab.stra["tmc", "pct.lt.DL"]  <- x.lt.detect.pct
}

if ("tmu" %in% meth.list)
{
  tab.stra["tmu", "irep"]    <- irep
  tab.stra["tmu", "method"]  <- "tmu"

  tab.stra["tmu", "sexlabel"]    <- sexlabel
  tab.stra["tmu", "agelabel"]    <- agelabel
  tab.stra["tmu", "Age.num"]     <- iage
  tab.stra["tmu", "Age.mea"]     <- Age.mea

  tab.stra["tmu","subset.type"] <- subset.type
  tab.stra["tmu", "n"]          <- x.n
  tab.stra["tmu", "pct.lt.DL"]  <- x.lt.detect.pct
}

if ("tuk" %in% meth.list)
{
  tab.stra["tuk", "irep"]    <- irep
  tab.stra["tuk", "method"]  <- "tuk"

  tab.stra["tuk", "sexlabel"]    <- sexlabel
  tab.stra["tuk", "agelabel"]    <- agelabel
  tab.stra["tuk", "Age.num"]     <- iage
  tab.stra["tuk", "Age.mea"]     <- Age.mea

  tab.stra["tuk","subset.type"] <- subset.type
  tab.stra["tuk", "n"]          <- x.n
  tab.stra["tuk", "pct.lt.DL"]  <- x.lt.detect.pct
}

# -------------------------------------------------------------------------
#  Analysis only if data subset has sufficient cases (absolute minimum)
#  Data distribution may additionally cause too few bins of required size - 
#  is checked below. Check here is not finally, values below detection limit
#  may play an additional role.

#  Required minimum number of observations in a stratum
x.n.min <- n.per.bin.min*x.tr.bins.min

if (x.n < x.n.min)
{ #  Not enough observations - no analysis
  cat("\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++",
      "\nData subset for    ", sexlabel,"  ",agelabel,
      "\n  has only         ", x.n," values",
      "\n  Required minimum:", x.n.min,
      "\n                   ==> no analysis",
      "\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n")
 
  #  Set switch and error message 
  ErrMsg[1,"Occurred"] <- TRUE
  go.on <- FALSE

}  else
{ 
  # ===========================================================================
  #  Required minimum number of observations in a stratum exists: do analysis

  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  100\n") }

  #  Save test data  
  #if (outname.stra == "Krebs_GGT_2_M_40-49")
  #if (outname.stra == "ALT_Bochum_7_stationaer_M_18-29")
  #{ 
  #  save(x, file=paste("../temp/", outname.stra, ".RData", sep="")) 
  #}
 
  #  Calculate descriptive quantities
  x.min <- min(x)
  x.max <- max(x)

  #  Nonparametric quantiles of the total dataset. Subsequent RL estimates 
  #  must lie in [x.Q1, x.Q2]

  x.Q1 <- Quantile(x,probs=RL1.p)
  x.Q2 <- Quantile(x,probs=RL2.p)

  #  Wie differenziert sind die Daten (wieviel verschiedene Werte) ?
  x.table <- table(x)

  x.val   <- as.numeric(names(x.table))
  x.val.n <- length(x.table)

  # .........................................................................
  #  Empirical distribution function

  x.emp.cdf <- c(0, cumsum(x.table)/x.n)
  x.emp.x   <- c(0, x.val)

  # ==========================================================================
  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  200\n") }

  #  Calculate kde
  KDE <- FindKDE(x, round.unit, kernel.n, 
                 fig100.001, subtitle, kdecol,
                 Q.lo=0.025, Q.hi=0.975, b.fact=1.05)
  x.kde      <- KDE[["x.kde"]]
  x.kde.mode <- KDE[["x.kde.mode"]]

  xsupp <- x.kde$x

  x.lt.mode.n <- sum(x <  x.kde.mode)
  x.ge.mode.n <- sum(x >= x.kde.mode)

  # ==========================================================================
  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  300\n") }

  #  Produce a reasonable histogram on the original scale, together with a 
  #  smoothed version

  HIST <- FindHisto(x, round.unit, detect.limits.max, x.table, x.val, x.kde, 
                    n.per.bin.min, bins.n.min, bins.n.max,
                    smooth.hist1, smooth.hist2, red.limit, xlabel, subtitle, 
                    bordercol1, histcol1, bordercol2, histcol2, 
                    bordercol3, histcol3, kdecol, fig100.002)

  x.hist.eqd     <- HIST[["x.hist.eqd"]]      # equidistant, raw
  x.hist.eqd.smo <- HIST[["x.hist.eqd.smo"]]  # equidistant, smoothed
  x.hist         <- HIST[["x.hist"]]          # collapsed, possibly smoothed
  x.reso.red     <- HIST[["x.reso.red"]]      # reduced data set

  #  Histogram construction may have left too few bins (==> x.hist == NA).
  x.hist.exists <- (length(x.hist) > 1)

  if (x.hist.exists)
  {
    bins.n.act <-  length(x.hist$counts)

    #  Which bin in x.hist contains x.kde.mode?
    x.kde.mode.idx <- FindModeIndex(x.hist$breaks, x.hist$counts, x.kde.mode)

    #  counts.range refers to x.hist
    counts.range <- paste(formatC(min(x.hist$counts), format="f", digits=0),
                        "-", 
                        formatC(max(x.hist$counts), format="f", digits=0), 
                        sep="")

    # =========================================================================
    if (print.log.message) { cat("%%%   TMC_seg100_Analysis  400\n") }

    cat("\n\n", sep.p,
    "\nStart of analysis                      ", format(time.start),
    "\nData file                              ",datafile, 
    "\nAnalyte                                ",xlabel, 
    "\nAnalysis of the stratum defined by",
    "\n  Age                                  ",agelabel,
    "\n  Sex                                  ",sexlabel,    
    "\nRecords for analysis in this stratum   ",x.n,
    "\nValues lie in the range                ",x.min, x.max,
    "\nNumber of distinct values              ",x.val.n,
    "\nSmoothing parameter for x              ",x.kde$bw,
    "\nmode(x)                                ",x.kde.mode,
    "\n", sep.p, 
    "\n\n")

    # ---------------------------------------------------------------------------
    if (print.log.message) { cat("%%%   TMC_seg100_Analysis  500\n") }

    #  Is there indication of problematic rounding (eg "go to the even value"
    #  or inconsistent rounding)?  Do diagnostic plot.

    if (plot.fig100.004)
    {
      dev.set(fig100.004)

      par(las=par.las)    
      par(tcl=par.tcl)    
      plot(as.numeric(names(x.table)), as.numeric(x.table),type="h",
         log="x",
         ylim=c(0,max(x.table)),
         xlab=xlabel, ylab="Count",
         main="Frequencies of observed values",
         sub=subtitle,cex.sub=0.7)
      # if (!eval.rep) savePlot(file=fig100.004.file,type=figtype)
    }

  }  else    #  x.hist.exists
  {
    #  x.hist does not exist
    bins.n.act <- 0
  } 

  # =========================================================================
  #  Start RL calculation

  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  600\n") }

  #  Initialize output tables 

  source("TMC_seg101_Analysis_Ini.R")

  # =========================================================================
  #  If group membership is known (typically for simulated data):
  #  Calculate the ideal direct result  
  #  Ideal direct result: all nonpathological persons are correctly identified
  #  No pathological cases lie in the truncation interval (though no 
  #  truncation interval is needed for RLx.npa)
  # 
  
  if (!is.na(spalte.g))
  {
    if (print.log.message) { cat("%%%   TMC_seg100_Analysis  700\n") }

    source("TMC_seg102_Analysis_npa.R")

  }  else
  {
    if (print.log.message) { cat("%%%   TMC_seg100_Analysis  800\n") }

    #  Output table tab.npa initialized in seg_1010 
  }

  # ==========================================================================
  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  900\n") }

  if (x.hist.exists)
  { 
    #  There is a reasonable number of bins
    #  Find TI candidates with required properties (proportion of values)
    #  bins.n.min in next call replaced by x.tr.bins.min 23.02.2021

    TI.can <- proportions(x.hist$counts, x.kde.mode.idx, 
                          x.tr.bins.min, x.tr.prop.min, x.tr.prop.max)

    #  Go through the candidate list in groups of x.tr.prop
    NoQqwSolution <- TRUE   
    NoTmcSolution <- TRUE
    first.cycle   <- TRUE   
    iti <- x.tr.prop.ints.n + 1     #  set in seg040

    while (NoTmcSolution & iti >= 2 )
    {
      iti <- iti - 1     # index for prop range

      x.tr.prop.range <- paste(formatC(100*x.tr.prop.limits[iti], width=2,
                                       digits=0, format="f"), "-",
                               formatC(100*x.tr.prop.limits[iti+1], width=3,
                                       digits=0, format="f"), "%", sep="")

      TI.subset <- (x.tr.prop.limits[iti] <= TI.can[ ,"prop"]) &
                   (TI.can[ ,"prop"] < x.tr.prop.limits[iti+1] )
      TI.subset.n <- sum(TI.subset, na.rm=TRUE)
          
      if (print.log.message) { cat("%%%   TMC_seg100_Analysis  910\n") }

      if (TI.subset.n > 0)
      {   
        #  Non-empty subset, use the prop.limits as preliminary 
        #  limits for QQW, TMC, TMU estimation

        #  Calculate QQW solution
        #  Use reduced and resolved dataset if the dataset is large

        if (print.log.message) 
        { cat("%%%   TMC_seg100_Analysis  920", date(), "\n") }

        source("TMC_seg104_Analysis_qqw.R")

        # ==================================================================
        if (print.log.message) { cat("%%%   TMC_seg100_Analysis 1100\n") }

        #  Calculate Bhattacharya solution, if requested. Needs equidistant 
        #  histogram.
          
        if ("bha" %in% meth.list)
        {  
          data.text <- xlabel
          tab.bha   <- Bhatta(x.hist.eqd.smo, x.kde, data.text, xlabel, subtitle,
                            RL1.p, RL2.p, 
                            fig100.013, fig100.014,
                            kdecol, bhacol, bordercol1, histcol1)
        }

        # ==================================================================
        if (print.log.message) 
        { cat("%%%   TMC_seg100_Analysis 1200", date(), "\n") }

        # If a QQW solution exists: TMC

        QqwSolution <- length(tab.qqw) > 1

        if ( QqwSolution )         
        {
          #  QQW solution exists.
          #  Calculate TMC solution. x.hist may be smoothed,
          #  depending on smooth.hist1 /2, but further processing is the same
          #  in all cases. tab.qqw contains truncation intervals and initial
          #  parameter estimates.
    
          #  Do TMC analysis for one TI subset group, find optimal among
          #  all actual size groups
          
          source("TMC_seg105_Analysis_tmc.R")

          #  No further x.tr.prop ranges if !NoTmcSolution 
     
          cat("%%%   TMC_seg100_Analysis 1235", date(), "\n")
        
        } #  QQW solution exists
      }   #  TI subset has values
    }     #  loop over TI subsets 

    # ------------------------------------------------------------------------
    #  Is there a TMC solution?
    
    if (print.log.message) 
    { cat("%%%   TMC_seg100_Analysis 1240", date(), "\n") }

    TmcSolution <- (length(tab.tmc) > 1) & (!is.na(tab.tmc[1]))

    if ( TmcSolution )
    {
      #  Replace pseudo-zero estimate by zero for nicer printout
      if (abs(tab.tmc["lambda.tmc"]) < fastnull) { tab.tmc["lambda.tmc"] <- 0 }

      # TMC Solution exists, plot result   

      if (print.log.message) { cat("%%%   TMC_seg100_Analysis 1700\n") }

      if (plot.fig100.016)
      { 
        #  Plot result - basic part
        #  No plot file name, save is done here below
        fig100.016.lim <- PlotHistFit(fig100.016, NA, figtype, 
                x.hist, xlabel, "TMC result", subtitle, n.per.bin.min,
                NA, x.val, counts.range,  
                paste(formatC(100*tab.tmc["x.tr.prop"], format="f", width=5, 
                      digits=1), "%", sep=""),
                x.clip.min, x.clip.max, x.clip.by1, x.clip.by2, 
                x.clip.type, NA, par.las, par.tcl,
                bordercol, histcol, difcol, kdecol, denlty)

        #  Plot result - tmc specific part 
        #  Add estimated tmc solution, RLs, TI
        #  Calculate estimated TMC density (not yet scaled by prev.c.tmc)
        xc.pdf.tmc <- pdf.PN(xsupp, tab.tmc["lambda.tmc"], 
                                   tab.tmc["mue.tmc"], tab.tmc["sigma.tmc"])    

        #  Calculate difference between x.kde and xc.pdf.tmc

        #  Truncate difference below to yplotmin
        #  Switched off 01.05.2021
        #x.kde.tmc <- x.kde$y -  tab.tmc["prev.c.tmc"] * xc.pdf.tmc
        #x.kde.tmc[x.kde.tmc < fig100.016.lim["yplotmin"]] <- 
        #                                           fig100.016.lim["yplotmin"]
  
        PlotMetRes(fig100.016, fig100.016.file, figtype, 
                   fig100.016.lim["yplotmin"],
                   xsupp, tab.tmc["prev.c.tmc"], xc.pdf.tmc, tmccol,
                   x.kde.mode, kdecol, NA, difcol, difbordercol, difhistcol, 
                   tab.tmc["x.RL1.tmc"], tab.tmc["x.RL2.tmc"], 
                   0.85*fig100.016.lim["yplotmax"], 
                   tab.tmc["x.tr.lo"], tab.tmc["x.tr.hi"], 
                   0.90*fig100.016.lim["yplotmin"], tmccol,
                   temp.tmc)

        # -------------------------------------------------------------
        if (print.log.message) { cat("%%%   TMC_seg100_Analysis 1800\n") }

        #  If the data is generated : show unaffected range
        if (!is.na(x.unaffected.lo) & !is.na(x.unaffected.hi))
        { 
          lines(c(x.unaffected.lo, x.unaffected.hi), 
                    0.60*fig100.016.lim["yplotmin"]*c(1,1), col="chartreuse", 
                    lwd=2)
        }

        # -------------------------------------------------------------
        # Save tmc result figure
        savePlot(file=fig100.016.file, type=figtype)
 
      }      #  if (plot.fig100.016)

      # ================================================================
      if (print.log.message) 
      { cat("%%%   TMC_seg100_Analysis 1900", date(), "\n") }

      if ("tmu" %in% meth.list)
      { #  Calculate TMU solution

        source("TMC_seg106_Analysis_tmu.R") 
 
      }  

      # ================================================================
      if (print.log.message) 
      { cat("%%%   TMC_seg100_Analysis 2000", date(), "\n") }           

      #  Calculate Tukey solution, using lambda from tmc

      if ("tuk" %in% meth.list)
      { #  Calculate Tukey solution

        source("TMC_seg108_Analysis_tuk.R")       

      }  

      if (print.log.message) 
      { cat("%%%   TMC_seg100_Analysis 2100", date(), "\n") }           

      # ================================================================

    }              #  TMC solution exists
  }                #  x.hist exists

  # ==========================================================================
  if (print.log.message) { cat("%%%   TMC_seg100_Analysis 2200\n") }
  # ==========================================================================

  #  Combine essential results from all methods for this stratum
  #  tab.stra defined in the beginning

  if ("npa" %in% meth.list)
  {
    tab.stra["npa", "x.tr.lo"] <- tab.npa["x.tr.lo.npa"]
    tab.stra["npa", "x.tr.hi"] <- tab.npa["x.tr.hi.npa"]
    tab.stra["npa", "RL1"]     <- tab.npa["x.RL1.npa"]
    tab.stra["npa", "RL2"]     <- tab.npa["x.RL2.npa"]
    tab.stra["npa", "prop"]    <- tab.npa["x.tr.prop.npa"]
    tab.stra["npa", "prev.l"]  <- tab.npa["prev.l.npa"]
    tab.stra["npa", "prev.c"]  <- tab.npa["prev.c.npa"]
    tab.stra["npa", "prev.r"]  <- tab.npa["prev.r.npa"]
  }

  if ( ("qqw" %in% meth.list) & !NoQqwSolution )
  {
    tab.stra["qqw", "x.tr.lo"] <- tab.qqw[1, "x.tr.lo"]
    tab.stra["qqw", "x.tr.hi"] <- tab.qqw[1, "x.tr.hi"]
    tab.stra["qqw", "bins.n"]  <- tab.qqw[1, "bins.n"]
    tab.stra["qqw", "prop"]    <- tab.qqw[1, "prop.qqw"]
    tab.stra["qqw", "lambda"]  <- tab.qqw[1, "lambda.qqw"]
    tab.stra["qqw", "mue"]     <- tab.qqw[1, "mue.qqw"]
    tab.stra["qqw", "sigma"]   <- tab.qqw[1, "sigma.qqw"]
    tab.stra["qqw", "RL1"]     <- tab.qqw[1, "x.RL1.qqw"]
    tab.stra["qqw", "RL2"]     <- tab.qqw[1, "x.RL2.qqw"]
    tab.stra["qqw", "RL1.cilo"] <- tab.qqw[1, "x.RL1.cilo"]
    tab.stra["qqw", "RL1.cihi"] <- tab.qqw[1, "x.RL1.cihi"]
    tab.stra["qqw", "RL2.cilo"] <- tab.qqw[1, "x.RL2.cilo"]
    tab.stra["qqw", "RL2.cihi"] <- tab.qqw[1, "x.RL2.cihi"]
    tab.stra["qqw", "bins.n"]  <- tab.qqw[1, "bins.n"]
    tab.stra["qqw", "x.tr.n"]  <- tab.qqw[1, "x.tr.n"]   # may refer to reduced
                                                         # histogram 
    tab.stra["qqw", "prop"]    <- tab.qqw[1, "prop.qqw"] # may refer to reduced
                                                         # histogram 
    tab.stra["qqw", "opt.crit"] <- unname(temp.qqw[["res"]]["opt.crit"])
    tab.stra["qqw", "p.fit"]   <- unname(temp.qqw[["res"]]["chi2.trun.p"])
    tab.stra["qqw", "prev.l"]  <- tab.qqw[1, "prev.l.qqw"]
    tab.stra["qqw", "prev.c"]  <- tab.qqw[1, "prev.c.qqw"]
    tab.stra["qqw", "prev.r"]  <- tab.qqw[1, "prev.r.qqw"]
  }

  if ("bha" %in% meth.list)
  {
    #  @@@ subpopulation auswhlen
    tab.stra["bha", "RL1"]    <- tab.bha[["subpop"]]["comp01", "RL1.2"]
    tab.stra["bha", "RL2"]     <- tab.bha[["subpop"]]["comp01", "RL2.2"]
  }

  if (("tmc" %in% meth.list) & x.hist.exists & TmcSolution)
  {
    tab.stra["tmc", "x.tr.lo"] <- tab.tmc["x.tr.lo"]
    tab.stra["tmc", "x.tr.hi"] <- tab.tmc["x.tr.hi"]
    tab.stra["tmc", "x.tr.n"]  <- tab.tmc["x.tr.n"]
    tab.stra["tmc", "bins.n"]  <- tab.tmc["ihi"] - tab.tmc["ilo"] + 1
    tab.stra["tmc", "prop"]    <- tab.tmc["x.tr.prop"]  
    tab.stra["tmc", "rc"]      <- tab.tmc["rc"]  
    tab.stra["tmc", "iter"]    <- tab.tmc["iter"]  
    tab.stra["tmc", "lambda"]  <- tab.tmc["lambda.tmc"]
    tab.stra["tmc", "mue"]     <- tab.tmc["mue.tmc"]
    tab.stra["tmc", "sigma"]   <- tab.tmc["sigma.tmc"]
    tab.stra["tmc", "RL1"]     <- tab.tmc["x.RL1.tmc"]
    tab.stra["tmc", "RL2"]     <- tab.tmc["x.RL2.tmc"]

    tab.stra["tmc", "RL1.cilo"] <- tab.tmc["x.RL1.cilo"]
    tab.stra["tmc", "RL1.cihi"] <- tab.tmc["x.RL1.cihi"]
    tab.stra["tmc", "RL2.cilo"] <- tab.tmc["x.RL2.cilo"]
    tab.stra["tmc", "RL2.cihi"] <- tab.tmc["x.RL2.cihi"]

    tab.stra["tmc", "opt.crit"] <- tab.tmc["opt.crit"]
    tab.stra["tmc", "p.fit"]   <- tab.tmc["p.fit"]
    tab.stra["tmc", "bins.n"]  <- tab.tmc["ihi"] - 
                                  tab.tmc["ilo"] + 1
    tab.stra["tmc", "prop"]    <- tab.tmc["x.tr.prop"]
    tab.stra["tmc", "prev.l"]  <- tab.tmc["prev.l.tmc"]
    tab.stra["tmc", "prev.c"]  <- tab.tmc["prev.c.tmc"]
    tab.stra["tmc", "prev.r"]  <- tab.tmc["prev.r.tmc"]
  }

  if ("tmu" %in% meth.list & x.hist.exists & TmcSolution)
  {
    tab.stra["tmu", "x.tr.lo"] <- tab.tmc["x.tr.lo"]
    tab.stra["tmu", "x.tr.hi"] <- tab.tmc["x.tr.hi"]
    tab.stra["tmu", "x.tr.n"]  <- tab.tmc["x.tr.n"]
    tab.stra["tmu", "bins.n"]  <- tab.tmc["ihi"] - tab.tmc["ilo"] + 1
    tab.stra["tmu", "prop"]    <- tab.tmc["x.tr.prop"]  
    tab.stra["tmu", "rc"]      <- tab.tmu["rc.tmu"]  
    tab.stra["tmu", "iter"]    <- tab.tmu["iter.tmu"]  
    tab.stra["tmu", "lambda"]  <- tab.tmc["lambda.tmc"]
    tab.stra["tmu", "mue"]     <- tab.tmu["mue.tmu"]
    tab.stra["tmu", "sigma"]   <- tab.tmu["sigma.tmu"]
    tab.stra["tmu", "RL1"]     <- tab.tmu["x.RL1.tmu"]
    tab.stra["tmu", "RL2"]     <- tab.tmu["x.RL2.tmu"]

    tab.stra["tmu", "RL1.cilo"] <- tab.tmu["x.RL1.cilo"]
    tab.stra["tmu", "RL1.cihi"] <- tab.tmu["x.RL1.cihi"]
    tab.stra["tmu", "RL2.cilo"] <- tab.tmu["x.RL2.cilo"]
    tab.stra["tmu", "RL2.cihi"] <- tab.tmu["x.RL2.cihi"]
    tab.stra["tmu", "opt.crit"] <- tab.tmu["opt.crit.tmu"]
    tab.stra["tmu", "p.fit"]   <- tab.tmu["p.fit.tmu"]
    tab.stra["tmu", "prev.l"]  <- tab.tmu["prev.l.tmu"]
    tab.stra["tmu", "prev.c"]  <- tab.tmu["prev.c.tmu"]
    tab.stra["tmu", "prev.r"]  <- tab.tmu["prev.r.tmu"]
  }                #  do tmu

  if ("tuk" %in% meth.list & x.hist.exists & TmcSolution)
  {
    tab.stra["tuk", "x.tr.lo"] <- tab.tuk["x.tr.lo"]
    tab.stra["tuk", "x.tr.hi"] <- tab.tuk["x.tr.hi"]
    tab.stra["tuk", "x.tr.n"]  <- tab.tuk["x.tr.n"]
    tab.stra["tuk", "bins.n"]  <- NA
    tab.stra["tuk", "prop"]    <- (x.n-tab.tuk["out.lo"]-tab.tuk["out.hi"])/ 
                                  x.n  
    tab.stra["tuk", "rc"]      <- NA 
    tab.stra["tuk", "iter"]    <- NA  
    tab.stra["tuk", "lambda"]  <- tab.tmc["lambda.tmc"]
    tab.stra["tuk", "mue"]     <- tab.tuk["mean"]
    tab.stra["tuk", "sigma"]   <- tab.tuk["sd.adj"]
    tab.stra["tuk", "RL1"]     <- tab.tuk["x.RL1.tuk"]
    tab.stra["tuk", "RL2"]     <- tab.tuk["x.RL2.tuk"]

    tab.stra["tuk", "RL1.cilo"] <- NA
    tab.stra["tuk", "RL1.cihi"] <- NA
    tab.stra["tuk", "RL2.cilo"] <- NA
    tab.stra["tuk", "RL2.cihi"] <- NA
    tab.stra["tuk", "opt.crit"] <- tab.tuk["opt.crit.tuk"]
    tab.stra["tuk", "p.fit"]    <- tab.tuk["p.fit.tuk"]
    tab.stra["tuk", "prev.l"]   <- tab.tuk["prev.l.tuk"]
    tab.stra["tuk", "prev.c"]   <- tab.tuk["prev.c.tuk"]
    tab.stra["tuk", "prev.r"]   <- tab.tuk["prev.r.tuk"]
  }                #  do tuk

  # --------------------------------------------------------------------------
  #  Error code for result. Considers only goodness of fit. Possible 
  #  extension: check estimated prevalence for reasonable values
  tab.stra[ , "errcode"] <- " ?"
  ok <- (tab.stra[ , "p.fit"] > p.fit.min)
  ok[is.na(ok)] <- FALSE
  tab.stra[ok, "errcode"] <- "ok"

  ok <- (tab.stra[ , "p.fit"] <= p.fit.min)
  ok[is.na(ok)] <- FALSE
  tab.stra[ok, "errcode"] <- "!!"

  # --------------------------------------------------------------------------
  #  Toleranzintervalle fr die tmc-RLs per bootstrapping berechnen
  #  NOT YET ADAPTED TO NEW CALL OF tmc.master

  source("TMC_seg109_TMC_BS.R")

  # --------------------------------------------------------------------------

  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  Position 5000\n") }

 
  #  ==================================================================
  #  Print results tables per stratum (not if replicates are analysed) 

  if (!eval.rep) { sink(file=outname.stra.txt,split=TRUE) }

  cat("\n\n", sep.e, 
      "\n\nRL ESTIMATES, ALL REQUESTED METHODS, ONE age-sex STRATUM",
      "\nUser                                   ", user,
      "\nData information taken from            ", info.file,
      "\nAnalysis date                          ", date(),
      "\nCopy of this table in                  ",
      "\n    ",                                   outname.stra.txt,
      "\nData file name                         ",
      "\n    ",                                   infile,
      "\nFilter  outpatient/hospitalized        ", use.oh,
      "\nFilter  device                         ", use.dev,
      "\nFilter  analysis time of day           ", paste(ana.hour.min, ":",
                                                         ana.minu.min, " - ",
                                                         ana.hour.max, ":",
                                                         ana.minu.max, sep=""),
      "\nSex group                              ", sexlabel,    
      "\nAge interval                           ", agelabel,
      "\nInput values are multiplied by         ", scale.fact,
      "\nScaled values are limited by           ", x.lo.limit,x.hi.limit,
      "\nScaled values are rounded here to      ",round.unit,
      "\nTotal number of all values in file     ",x.n1,
      "\nTotal number of numerical values       ",x.n14,
      "\nNumber of values in this stratum       ",x.n,
      "\nNumber of distinct values in stratum   ",x.val.n,
      "\nMaximal detection/determination limit  ",detect.limits.max,
      "\n# of values / % below detection limit  ",x.lt.detect.n, " / ",
                                                  format(x.lt.detect.pct,digits=3),
                                                  "%",
      "\nValue range                            ",x.min, x.max,
      "\nEmpirical P025, P975, complete stratum ",x.Q1, x.Q2,
      "\n",
      "\nSmooth empirical histogram?            ",smooth.hist1, smooth.hist2,
      "\nMin. # of values per bin               ",n.per.bin.min,
      "\nMin. # of bins in stratum              ",bins.n.min,     
      "\nMin. # of bins in truncation interval  ",x.tr.bins.min,     
      "\nMin. # of values in stratum            ",x.n.min,     
      "\nActual # of bins in stratum            ",bins.n.act,
      "\nUser-specified min. prop. in TI        ", x.tr.prop.min, 
      "\nUser-specified max. prop. in TI        ", x.tr.prop.max, 
      "\nWeight of truncation interval length   ",l.fact,
      "\nPenalty factor 'wrong prevalence'      ",p.fact,
      "\nPenalty factor 'wrong prediction'      ",w.fact,
      "\nMinimal lambda                         ",lambda.min,
      "\nMaximal lambda                         ",lambda.max,
      "\n",
      "\nTMC: Truncation interval               ",tab.tmc["x.tr.lo"], 
                                                  tab.tmc["x.tr.hi"],   
      "\nTMC: TI contains proportion ...        ",tab.tmc["x.tr.prop"],
      "\nTMC: estimated lambda                  ",tab.tmc["lambda.tmc"],
      "\nTMC: estimated mu                      ",tab.tmc["mue.tmc"],
      "\nTMC: estimated sigma                   ",tab.tmc["sigma.tmc"],
      "\nTMC: chi^2 from truncation interval    ",format(
                                                    tab.tmc["chi2.trun"],
                                                    digits=4),
      "\nTMC: chi^2 from outside TI             ",format(
                                                   tab.tmc["chi2.total"]-
                                                   tab.tmc["chi2.trun"],
                                                   digits=4),
      "\n",
      "\nFalse negative probability  <RL1 / <RL2", RL1.p,"/ ",RL2.p,
      "\nTMC: estimated RLs                       RL1 = ", 
                                             format(tab.tmc["x.RL1.tmc"],
                                                    digits=4),      
      "\n                                         RL2 = ", 
                                             format(tab.tmc["x.RL2.tmc"],
                                                    digits=4), 
      "\nAssessment of TMC solution",
      "\np for chi^2 goodness of fit            ",format(
                                                    tab.tmc["p.fit"],
                                                    digits=4), 
      "\nRL1 >= x.Q1, RL2 <= x.Q2               ", crit1, crit2,
      "\nprev.l ok / prev.c ok /prev.r ok?      ", crit3, crit4, crit5,
      "\nError structure ok?                    ", crit6,
      "\nTotal score for solution (max =",5+crit123456,")    ", 
                                                tab.tmc["sol.score"],    
      "\n", sep.e, 
      "\n\n")


  if (any(ErrMsg[ ,"Occurred"]))
  { cat("\n +++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n")
    print(ErrMsg[ErrMsg[ ,"Occurred"], "ErrTxt"])
    cat(   "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n")
  } 

  if (user == "WW")
  { #  Ausfhrung fr WW

    #cat("\n All details \n")
    #print(tab.stra[meth.list, ])
    #cat("\n")
  }

  if (!eval.rep) { sink() }
 
  # ==========================================================================
  if (print.log.message) { cat("%%%   TMC_seg100_Analysis  Position 6000\n") }
  # ==========================================================================

}  #  if (x.n < x.n.min)

# ---------------------------------------------------------------------------
#  Copy results in tab.stra (one stratum) to tab.f (one file)
#  Preliminary: gtab instead of tab.f

# ==========================================================================
if (print.log.message) { cat("%%%   TMC_seg100_Analysis  Position 6100\n") }
# ==========================================================================

#  block.start controls the identification by sexlabel and agelabel in the 
#  printout

#  Put only rows with information in gtab

if (!is.na(tab.stra[1, "irep"]) )
{
  block.start          <- TRUE
  iblock               <- iblock + 1
  for (meth in gtab.list)
  { igtab              <- igtab + 1

    meth.idx <- tab.stra[ ,"method"] == meth
  
    #  Append tab.stra to gtab
    if (iblock == 1 & igtab == 1)
    {
      #  First block at all, establish gtab
      gtab <- tab.stra[meth.idx, ]
    }  else
    {
      #  Append block to gtab
      gtab <- rbind(gtab, tab.stra[meth.idx, ])
    }

    if (block.start)
    { 
      gtab[igtab,"Sex"]     <- sexlabel
      gtab[igtab,"Age"]     <- agelabel
    } else 
    { 
      gtab[igtab,"Sex"]     <- " "
      gtab[igtab,"Age"]     <- " "
    }
    block.start          <- FALSE

    # ==========================================================================
    if (print.log.message) { cat("%%%   TMC_seg100_Analysis  Position 6400\n") }
    # ==========================================================================

  }    #  information for stratum is available
  cat("\n", sep.e, 
    "\n   End analysis by seg100 for", 
    "\n   ", outname.stra,
    "\n", sep.e, 
    "\n", sep="")
   
}      #  x.n > x.n.min

# ============================================================================
if (print.log.message) { cat("%%%   TMC_seg100_Analysis  End\n") }
# ============================================================================

#############################################################################
