#  TMC_seg202_SimResults.R
#  (c) wwosniok@math.uni-bremen.de

#  Presentation of results if a sequence of data files is evaluated, 
#  typically when analysing simulated data

#  INPUT:
#  soutfile.tmc.csv   contains results from bha, npa, qqw, tmc, tmu
#                     usually produced by TMC
#  soutfile.tml.csv   contains results from tml. Produced by FAs
#                     RLE, modified for this output. May not exist.

#  OUTPUT
#  outname.scen.rep.csv   contains results from all methods, per replicate
#  outname.scen.sum.csv   summary over all replicates, per method


#  CHANGE HISTORY
#  17.04.2021 csv output files for a scenario new organized. New file names,
#             the *.rep file has only results specific to a replicate,
#             the *.sum file has all control values
#  11.12.2020 New organisation with figure production by dedicated segments/
#             functions. See TMC4 logbook, ch. 7.2 .
#
# ===========================================================================
if (print.log.message) { cat("%%%   TMC_seg202_SimResults  Start\n") }
# ===========================================================================

source("TMC_seg201_OpenWindows.R")

#  Results from TMC are still in 'stab' with a copy in soutfile.tmc.csv
#  Read this file for easier development and put in stab.TMC
#  'stab' will lateron be used for joining results from all methods
#  '.TMC'  indicates that the data frame is calculated by TMC (but usually
#  contains more than method=tmc).
 
stab.TMC <- read.table(file=soutfile.tmc.csv, header=TRUE, sep=";", dec=".",
                       stringsAsFactors=FALSE)

#  Construct meth.list.202 from stab.TMC
meth.list.202 <- sort(unique(stab.TMC[ ,"method"]))
meth.list.202.n <- length(meth.list.202)

#  Is there a result file from TML? If so, join with stab.TMC to stab
if (file.exists(soutfile.tml.csv))  
{ # yes, read it
  stab.tml0 <- read.table(file=soutfile.tml.csv, header=TRUE, sep=";", dec=".",
                         stringsAsFactors=FALSE)
  stab.tml0.nrow <- nrow(stab.tml0)

  #  Join TMC and TML as stab, noting that tml has less variables than stab
  stab     <- stab.TMC
  stab.tml <- data.frame(matrix(NA, nrow=stab.tml0.nrow, ncol=ncol(stab)))
  colnames(stab.tml)     <- colnames(stab)
  stab.tml[ , "irep"]    <- stab.tml0[ ,"irep"]
  stab.tml[ , "method"]  <- "tml"
  stab.tml[ , "RL1"]     <- stab.tml0[ ,"RL1"]
  stab.tml[ , "RL2"]     <- stab.tml0[ ,"RL2"]
  stab.tml[ , "lambda"]  <- stab.tml0[ ,"lambda"]
  stab.tml[ , "mue"]     <- stab.tml0[ ,"mue"]
  stab.tml[ , "sigma"]   <- stab.tml0[ ,"sigma"]

  stab <- rbind(stab,stab.tml)
  stab <- stab[order(stab[ ,"irep"], stab[ ,"method"]), ]
 
  meth.list.202   <- c(meth.list.202, "tml")
}

# ----------------------------------------------------------------------------
#  Select methods to display. Will be moved to seg015 later.
#  Exclude bha because too preliminary

ok   <- stab[ , "method"] != "bha"
stab <- stab[ok, ]
meth.list.202 <- unique(stab[ , "method"]) 
meth.list.202.n <- length(meth.list.202)

# ----------------------------------------------------------------------------
#  Create summarizing table over all replicates, one row per method

scen.sum.colnames <- c("n.per.bin.min","prop.qqw",
                     "prop.mw","prop.sd","prop.min","prop.max",
                     "RL1.mw","RL1.med","RL1.sd","RL1.min","RL1.max",
                     "RL2.mw","RL2.med","RL2.sd","RL2.min","RL2.max",
                     "cov.RL1RL2",
                     "prev.l.mw","prev.l.sd","prev.l.min","prev.l.max",
                     "prev.c.mw","prev.c.sd","prev.c.min","prev.c.max",
                     "prev.r.mw","prev.r.sd","prev.r.min","prev.r.max",
                     "lambda.mw","lambda.sd","lambda.min","lambda.max",
                     "mue.mw","mue.sd","mue.min","mue.max",
                     "sig.mw","sig.sd","sig.min","sig.max",
                     "cov.muesig",
                     "p.fit.mw","p.fit.sd","p.fit.min","p.fit.max")
scen.sum <- matrix(NA,nrow=meth.list.202.n,ncol=length(scen.sum.colnames))
rownames(scen.sum) <- meth.list.202
colnames(scen.sum) <- scen.sum.colnames

#  Find blocks with missing estimates. If too few data points or bins,
#  tmc result (and others) is missing.

is.ok <- rep(TRUE, times=nrep*meth.list.202.n)

for (i in 1+seq(0,(nrep-1)*meth.list.202.n, by=meth.list.202.n))
{ 
  idx <- i:(i+meth.list.202.n-1)
  if (any(is.na(stab[idx, "RL1"]))) 
  {
    is.ok[idx] <- FALSE
  }
}

#  Reduce stab accordingly
stab1 <- stab[is.ok, ]
nrep1 <- nrow(stab1)/meth.list.202.n

cat("\n Planned # of replicates ", nrep,
    "\n Successful estimates    ", nrep1,
    "\n")

subtitle.202 <- paste(RunId, " / ",datafile0, "_", spalte.w,
                     " (r",r.start," - r",r.ende,") / n=", x.n, sep="")

# ----------------------------------------------------------------------------
#  More additional measures describing the estimate
#  Calculate the False Positive Rates per method
#  FPR refers to the generating parameters. There is no way to use npa as
#  reference, because this method generates no distribution parameters.

FPR1    <- rep(NA, times=nrow(stab1))
FPR2    <- FPR1
FPR.sum <- FPR1

FPR1               <- cdf.PN(stab1[ ,  "RL1"],
                                 lambda.gen, yc.mode.gen, yc.sig.gen)
FPR2               <- 1 - cdf.PN(stab1[ ,  "RL2"],
                                 lambda.gen, yc.mode.gen, yc.sig.gen)
FPR.sum            <- abs(FPR1) + abs(FPR2)
stab1[ , "FPR1"]    <- FPR1
stab1[ , "FPR2"]    <- FPR2
stab1[ , "FPR.sum"] <- FPR.sum

# ============================================================================
#  Data is organized.

# ----------------------------------------------------------------------------
#  Fig202a: Joint distribution of RL1, RL2 in original units

source("TMC_seg202_SimResultsA.R")

# ----------------------------------------------------------------------------
#  Fig202b: Joint distribution of RL1, RL2 expressed as % of the npa results

source("TMC_seg202_SimResultsB.R")

# ----------------------------------------------------------------------------
#  Fig202c: Joint distribution of RL1, RL2 expressed as % of the npa results

source("TMC_seg202_SimResultsC.R")

# ----------------------------------------------------------------------------
#  More approaches for graphical presentation: see seg201

# -------------------------------------------------------------------------
cat("\nZusammenfassung der Ergebnisse aus allen Replikationen",
    "\n   round.unit      ", round.unit,
    "\n   smooth.hist1    ", smooth.hist1, 
    "\n   smooth.hist2    ", smooth.hist2, 
    "\n   l.fact          ", l.fact    ,
    "\n   p.fact          ", p.fact    ,
    "\n   r.fact          ", r.fact    ,
    "\n   s.fact          ", s.fact    ,
    "\n   w.fact          ", w.fact    ,
    "\n   n.per.bin.min   ", n.per.bin.min,
    "\n   bins.n.min      ", bins.n.min,
    "\n   bins.n.max      ", bins.n.max,
    "\n   x.tr.prop.min   ", x.tr.prop.min,
    "\n   x.tr.prop.max   ", x.tr.prop.max,
    "\n   p.fit.min       ", p.fit.min,
    "\n   crit6.p         ", crit6.p,
    "\n\n") 
print(scen.sum)


#  Einzelergebnisse als csv ("method" ist bereits eine Spalte,
#  Zeilennamen sind laufende Nr., nicht rausschreiben))
write.table(stab1,file=outname.scen.rep.csv,quote=FALSE,sep=";",dec=".",  
            append=FALSE, row.names=FALSE)
cat("\nResults per replication data set written to",
    "\n",outname.scen.rep.csv,
    "\n")

#  Damit die csv-Datei die richtigen Spaltennamen bekommt:
#  Zeilennamen als erste Spalte mit Namen "method" einbauen, dann
#  ohne Zeilennamen rausschreiben (Zeilennamen erzeugen Spalte ohne
#  Spaltennamen)
scen.sum.csv <- scen.sum
scen.sum.csv <- data.frame("method"=row.names(scen.sum),scen.sum)
scen.sum.csv.nrow <- nrow(scen.sum.csv)
scen.sum.csv <- data.frame(time.start=rep(time.start,times=scen.sum.csv.nrow),
                           r.start=rep(r.start,times=scen.sum.csv.nrow),
                           r.ende=rep(r.ende,times=scen.sum.csv.nrow),
                           round.unit=rep(round.unit,times=scen.sum.csv.nrow),
                           l.fact=rep(l.fact,times=scen.sum.csv.nrow),
                           p.fact=rep(p.fact,times=scen.sum.csv.nrow),
                           w.fact=rep(w.fact,times=scen.sum.csv.nrow),
                           n.per.bin.min=rep(n.per.bin.min,
                                             times=scen.sum.csv.nrow),
                           scen.sum.csv) 

write.table(scen.sum.csv,file=outname.scen.sum.csv,append=TRUE,
              quote=FALSE,sep=";",dec=".",row.names=FALSE) 

cat("\nSummary over all replication datasets written to",
      "\n",outname.scen.sum.csv,
      "\n\n")

# ===========================================================================
if (print.log.message) { cat("%%%   TMC_seg202_SimResults  End\n") }
# ===========================================================================

##############################################################################